"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from abbyy_document_ai.types import BaseModel
from datetime import date
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class BillingPeriodTypedDict(TypedDict):
    start_date: NotRequired[date]
    r"""The billing period start date"""
    end_date: NotRequired[date]
    r"""The billing period end date"""


class BillingPeriod(BaseModel):
    start_date: Annotated[Optional[date], pydantic.Field(alias="startDate")] = None
    r"""The billing period start date"""

    end_date: Annotated[Optional[date], pydantic.Field(alias="endDate")] = None
    r"""The billing period end date"""


class IssuerTypedDict(TypedDict):
    name: NotRequired[str]
    r"""Name of the company providing the services"""
    address: NotRequired[str]
    r"""Address of the company providing the services"""
    tax_id: NotRequired[str]
    r"""Tax ID of the company providing the services"""


class Issuer(BaseModel):
    name: Optional[str] = None
    r"""Name of the company providing the services"""

    address: Optional[str] = None
    r"""Address of the company providing the services"""

    tax_id: Annotated[Optional[str], pydantic.Field(alias="taxId")] = None
    r"""Tax ID of the company providing the services"""


class UtilityBillFieldsCustomerTypedDict(TypedDict):
    account_number: NotRequired[str]
    r"""Customer account number"""
    name: NotRequired[str]
    r"""Customer name"""
    customer_address: NotRequired[str]
    r"""Customer billing address"""
    service_address: NotRequired[str]
    r"""Address where the service was performed"""
    tax_id: NotRequired[str]
    r"""Customer tax ID"""


class UtilityBillFieldsCustomer(BaseModel):
    account_number: Annotated[Optional[str], pydantic.Field(alias="accountNumber")] = (
        None
    )
    r"""Customer account number"""

    name: Optional[str] = None
    r"""Customer name"""

    customer_address: Annotated[
        Optional[str], pydantic.Field(alias="customerAddress")
    ] = None
    r"""Customer billing address"""

    service_address: Annotated[
        Optional[str], pydantic.Field(alias="serviceAddress")
    ] = None
    r"""Address where the service was performed"""

    tax_id: Annotated[Optional[str], pydantic.Field(alias="taxId")] = None
    r"""Customer tax ID"""


class RemitToTypedDict(TypedDict):
    name: NotRequired[str]
    r"""Name of the business unit or payment recipient"""
    address: NotRequired[str]
    r"""Address of the business unit or payment recipient"""


class RemitTo(BaseModel):
    name: Optional[str] = None
    r"""Name of the business unit or payment recipient"""

    address: Optional[str] = None
    r"""Address of the business unit or payment recipient"""


class UtilityBillFieldsLineItemsTypedDict(TypedDict):
    billing_number: NotRequired[str]
    r"""The customer account number used to identify the account"""
    description: NotRequired[str]
    r"""A description of the line item"""
    quantity: NotRequired[float]
    r"""The number of units purchased"""
    unit_price: NotRequired[float]
    r"""The price of one item"""
    uom: NotRequired[str]
    r"""The unit of measurement for the line items"""
    amount: NotRequired[float]
    r"""The payment amount"""


class UtilityBillFieldsLineItems(BaseModel):
    billing_number: Annotated[Optional[str], pydantic.Field(alias="billingNumber")] = (
        None
    )
    r"""The customer account number used to identify the account"""

    description: Optional[str] = None
    r"""A description of the line item"""

    quantity: Optional[float] = None
    r"""The number of units purchased"""

    unit_price: Annotated[Optional[float], pydantic.Field(alias="unitPrice")] = None
    r"""The price of one item"""

    uom: Optional[str] = None
    r"""The unit of measurement for the line items"""

    amount: Optional[float] = None
    r"""The payment amount"""


class AdjustmentsTypedDict(TypedDict):
    adjustments: NotRequired[float]
    r"""Amount of the adjustment"""


class Adjustments(BaseModel):
    adjustments: Optional[float] = None
    r"""Amount of the adjustment"""


class UtilityBillFieldsTypedDict(TypedDict):
    r"""Fields extracted from a utility bill"""

    billing_period: NotRequired[BillingPeriodTypedDict]
    bill_date: NotRequired[date]
    r"""The date on which the bill was issued"""
    bill_number: NotRequired[str]
    r"""The bill number (also sometimes named Invoice Number)"""
    issuer: NotRequired[IssuerTypedDict]
    customer: NotRequired[UtilityBillFieldsCustomerTypedDict]
    remit_to: NotRequired[RemitToTypedDict]
    amount_due: NotRequired[float]
    r"""The total amount due for the billing period"""
    current_charges: NotRequired[float]
    r"""Charges for the current period"""
    currency: NotRequired[str]
    r"""The currency of the amount due"""
    line_items: NotRequired[List[UtilityBillFieldsLineItemsTypedDict]]
    previous_balance: NotRequired[float]
    r"""The balance for the previous billing period"""
    last_payment: NotRequired[float]
    r"""The payment made for the previous billing period"""
    balance_forward: NotRequired[float]
    r"""The balance brought forward from the previous billing period"""
    due_date: NotRequired[date]
    r"""The bill due date"""
    amount_after_due_date: NotRequired[float]
    r"""The amount charged if the payment is made after the due date"""
    late_charge: NotRequired[float]
    r"""Additional charges levied if payment is made after the due date"""
    penalties: NotRequired[float]
    r"""Charges levied on unpaid amounts for the billing period"""
    adjustments: NotRequired[List[AdjustmentsTypedDict]]
    r"""Payment adjustments"""


class UtilityBillFields(BaseModel):
    r"""Fields extracted from a utility bill"""

    billing_period: Annotated[
        Optional[BillingPeriod], pydantic.Field(alias="billingPeriod")
    ] = None

    bill_date: Annotated[Optional[date], pydantic.Field(alias="billDate")] = None
    r"""The date on which the bill was issued"""

    bill_number: Annotated[Optional[str], pydantic.Field(alias="billNumber")] = None
    r"""The bill number (also sometimes named Invoice Number)"""

    issuer: Optional[Issuer] = None

    customer: Optional[UtilityBillFieldsCustomer] = None

    remit_to: Annotated[Optional[RemitTo], pydantic.Field(alias="remitTo")] = None

    amount_due: Annotated[Optional[float], pydantic.Field(alias="amountDue")] = None
    r"""The total amount due for the billing period"""

    current_charges: Annotated[
        Optional[float], pydantic.Field(alias="currentCharges")
    ] = None
    r"""Charges for the current period"""

    currency: Optional[str] = None
    r"""The currency of the amount due"""

    line_items: Annotated[
        Optional[List[UtilityBillFieldsLineItems]], pydantic.Field(alias="lineItems")
    ] = None

    previous_balance: Annotated[
        Optional[float], pydantic.Field(alias="previousBalance")
    ] = None
    r"""The balance for the previous billing period"""

    last_payment: Annotated[Optional[float], pydantic.Field(alias="lastPayment")] = None
    r"""The payment made for the previous billing period"""

    balance_forward: Annotated[
        Optional[float], pydantic.Field(alias="balanceForward")
    ] = None
    r"""The balance brought forward from the previous billing period"""

    due_date: Annotated[Optional[date], pydantic.Field(alias="dueDate")] = None
    r"""The bill due date"""

    amount_after_due_date: Annotated[
        Optional[float], pydantic.Field(alias="amountAfterDueDate")
    ] = None
    r"""The amount charged if the payment is made after the due date"""

    late_charge: Annotated[Optional[float], pydantic.Field(alias="lateCharge")] = None
    r"""Additional charges levied if payment is made after the due date"""

    penalties: Optional[float] = None
    r"""Charges levied on unpaid amounts for the billing period"""

    adjustments: Optional[List[Adjustments]] = None
    r"""Payment adjustments"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .inputsource import InputSource, InputSourceTypedDict
from abbyy_document_ai.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class BeginDeliveryNoteFieldExtractionRequestBodyTypedDict(TypedDict):
    r"""Request to process a document using the delivery-note model"""

    input_source: InputSourceTypedDict
    r"""The source of the document(s) to be processed"""


class BeginDeliveryNoteFieldExtractionRequestBody(BaseModel):
    r"""Request to process a document using the delivery-note model"""

    input_source: Annotated[InputSource, pydantic.Field(alias="inputSource")]
    r"""The source of the document(s) to be processed"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from abbyy_document_ai.types import BaseModel
from datetime import date
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ShipperTypedDict(TypedDict):
    account_number: NotRequired[str]
    r"""The shipper's account number"""
    name: NotRequired[str]
    r"""The shipper's name"""
    address: NotRequired[str]
    r"""The shipper's address"""
    country: NotRequired[str]
    r"""The shipper's country"""
    city: NotRequired[str]
    r"""The shipper's city"""
    state_or_province: NotRequired[str]
    r"""The shipper's state or province"""
    postal_code: NotRequired[str]
    r"""The shipper's postal code"""
    street: NotRequired[str]
    r"""The shipper's street address"""
    tax_id: NotRequired[str]
    r"""The shipper's international tax identification number"""


class Shipper(BaseModel):
    account_number: Annotated[Optional[str], pydantic.Field(alias="accountNumber")] = (
        None
    )
    r"""The shipper's account number"""

    name: Optional[str] = None
    r"""The shipper's name"""

    address: Optional[str] = None
    r"""The shipper's address"""

    country: Optional[str] = None
    r"""The shipper's country"""

    city: Optional[str] = None
    r"""The shipper's city"""

    state_or_province: Annotated[
        Optional[str], pydantic.Field(alias="stateOrProvince")
    ] = None
    r"""The shipper's state or province"""

    postal_code: Annotated[Optional[str], pydantic.Field(alias="postalCode")] = None
    r"""The shipper's postal code"""

    street: Optional[str] = None
    r"""The shipper's street address"""

    tax_id: Annotated[Optional[str], pydantic.Field(alias="taxId")] = None
    r"""The shipper's international tax identification number"""


class ConsigneeTypedDict(TypedDict):
    account_number: NotRequired[str]
    r"""The consignee's account number"""
    name: NotRequired[str]
    r"""The consignee's name"""
    address: NotRequired[str]
    r"""The consignee's address"""
    country: NotRequired[str]
    r"""The consignee's country"""
    state_or_province: NotRequired[str]
    r"""The consignee's state or province"""
    city: NotRequired[str]
    r"""The consignee's city"""
    postal_code: NotRequired[str]
    r"""The consignee's postal code"""
    street: NotRequired[str]
    r"""The consignee's street address"""
    tax_id: NotRequired[str]
    r"""The consignee's international tax identification number"""


class Consignee(BaseModel):
    account_number: Annotated[Optional[str], pydantic.Field(alias="accountNumber")] = (
        None
    )
    r"""The consignee's account number"""

    name: Optional[str] = None
    r"""The consignee's name"""

    address: Optional[str] = None
    r"""The consignee's address"""

    country: Optional[str] = None
    r"""The consignee's country"""

    state_or_province: Annotated[
        Optional[str], pydantic.Field(alias="stateOrProvince")
    ] = None
    r"""The consignee's state or province"""

    city: Optional[str] = None
    r"""The consignee's city"""

    postal_code: Annotated[Optional[str], pydantic.Field(alias="postalCode")] = None
    r"""The consignee's postal code"""

    street: Optional[str] = None
    r"""The consignee's street address"""

    tax_id: Annotated[Optional[str], pydantic.Field(alias="taxId")] = None
    r"""The consignee's international tax identification number"""


class CarriersAgentTypedDict(TypedDict):
    account_number: NotRequired[str]
    r"""The carrier's account number"""
    iata_code: NotRequired[str]
    r"""The carrier's identification number"""
    name: NotRequired[str]
    r"""The carrier's name"""
    address: NotRequired[str]
    r"""The carrier's address"""
    country: NotRequired[str]
    r"""The carrier's country"""
    state_or_province: NotRequired[str]
    r"""The carrier's state or province"""
    city: NotRequired[str]
    r"""The carrier's city"""
    postal_code: NotRequired[str]
    r"""The carrier's postal code"""
    street: NotRequired[str]
    r"""The carrier's street address"""
    tax_id: NotRequired[str]
    r"""The carrier's international tax identification number"""


class CarriersAgent(BaseModel):
    account_number: Annotated[Optional[str], pydantic.Field(alias="accountNumber")] = (
        None
    )
    r"""The carrier's account number"""

    iata_code: Annotated[Optional[str], pydantic.Field(alias="iataCode")] = None
    r"""The carrier's identification number"""

    name: Optional[str] = None
    r"""The carrier's name"""

    address: Optional[str] = None
    r"""The carrier's address"""

    country: Optional[str] = None
    r"""The carrier's country"""

    state_or_province: Annotated[
        Optional[str], pydantic.Field(alias="stateOrProvince")
    ] = None
    r"""The carrier's state or province"""

    city: Optional[str] = None
    r"""The carrier's city"""

    postal_code: Annotated[Optional[str], pydantic.Field(alias="postalCode")] = None
    r"""The carrier's postal code"""

    street: Optional[str] = None
    r"""The carrier's street address"""

    tax_id: Annotated[Optional[str], pydantic.Field(alias="taxId")] = None
    r"""The carrier's international tax identification number"""


class AirlineTypedDict(TypedDict):
    name: NotRequired[str]
    r"""The name of the airline engaged to carry the goods"""
    address: NotRequired[str]
    r"""The airline's address"""
    country: NotRequired[str]
    r"""The airline's country"""
    state_or_province: NotRequired[str]
    r"""The airline's state or province"""
    city: NotRequired[str]
    r"""The airline's city"""
    postal_code: NotRequired[str]
    r"""The airline's postal code"""
    street: NotRequired[str]
    r"""The airline's street address"""


class Airline(BaseModel):
    name: Optional[str] = None
    r"""The name of the airline engaged to carry the goods"""

    address: Optional[str] = None
    r"""The airline's address"""

    country: Optional[str] = None
    r"""The airline's country"""

    state_or_province: Annotated[
        Optional[str], pydantic.Field(alias="stateOrProvince")
    ] = None
    r"""The airline's state or province"""

    city: Optional[str] = None
    r"""The airline's city"""

    postal_code: Annotated[Optional[str], pydantic.Field(alias="postalCode")] = None
    r"""The airline's postal code"""

    street: Optional[str] = None
    r"""The airline's street address"""


class AirWaybillNumberTypedDict(TypedDict):
    mawb_number: NotRequired[str]
    r"""The master air waybill number"""
    hawb_number: NotRequired[str]
    r"""The house air waybill number"""


class AirWaybillNumber(BaseModel):
    mawb_number: Annotated[Optional[str], pydantic.Field(alias="mawbNumber")] = None
    r"""The master air waybill number"""

    hawb_number: Annotated[Optional[str], pydantic.Field(alias="hawbNumber")] = None
    r"""The house air waybill number"""


class AirportOfDepartureTypedDict(TypedDict):
    airport_code: NotRequired[str]
    r"""The airport code of departure"""
    city: NotRequired[str]
    r"""The city of departure"""
    country: NotRequired[str]
    r"""The country of departure"""
    name: NotRequired[str]
    r"""The name of the departure airport"""


class AirportOfDeparture(BaseModel):
    airport_code: Annotated[Optional[str], pydantic.Field(alias="airportCode")] = None
    r"""The airport code of departure"""

    city: Optional[str] = None
    r"""The city of departure"""

    country: Optional[str] = None
    r"""The country of departure"""

    name: Optional[str] = None
    r"""The name of the departure airport"""


class AirportOfDestinationTypedDict(TypedDict):
    airport_code: NotRequired[str]
    r"""The airport code of destination"""
    city: NotRequired[str]
    r"""The city of destination"""
    country: NotRequired[str]
    r"""The country of destination"""
    name: NotRequired[str]
    r"""The name of the destination airport"""


class AirportOfDestination(BaseModel):
    airport_code: Annotated[Optional[str], pydantic.Field(alias="airportCode")] = None
    r"""The airport code of destination"""

    city: Optional[str] = None
    r"""The city of destination"""

    country: Optional[str] = None
    r"""The country of destination"""

    name: Optional[str] = None
    r"""The name of the destination airport"""


class FlightAndDateTypedDict(TypedDict):
    flight: NotRequired[str]
    r"""The number of the flight carrying the goods"""
    flight_date: NotRequired[date]
    r"""The date of the flight carrying the goods"""


class FlightAndDate(BaseModel):
    flight: Optional[str] = None
    r"""The number of the flight carrying the goods"""

    flight_date: Annotated[Optional[date], pydantic.Field(alias="flightDate")] = None
    r"""The date of the flight carrying the goods"""


class AirWaybillFieldsLineItemsTypedDict(TypedDict):
    quantity: NotRequired[float]
    r"""The number of pieces"""
    quantity_uom: NotRequired[str]
    r"""The unit of measurement used for the goods"""
    gross_weight_kg: NotRequired[float]
    r"""The weight of the goods in kilograms"""
    gross_weight_lb: NotRequired[float]
    r"""The weight of the goods in pounds"""
    rate_class: NotRequired[str]
    r"""The rate class of the goods"""
    commodity_item_no: NotRequired[str]
    r"""The code of the goods"""
    chargeable_weight: NotRequired[float]
    r"""The chargeable weight of the goods"""
    rate_or_charge: NotRequired[float]
    r"""Additional charges"""
    total: NotRequired[float]
    r"""The total amount of charges"""
    length: NotRequired[float]
    r"""The length of the package"""
    width: NotRequired[float]
    r"""The width of the package"""
    height: NotRequired[float]
    r"""The height of the package"""
    dimensions_uom: NotRequired[str]
    r"""The unit of measurement used for the dimensions"""
    description: NotRequired[str]
    r"""A description of the goods"""


class AirWaybillFieldsLineItems(BaseModel):
    quantity: Optional[float] = None
    r"""The number of pieces"""

    quantity_uom: Annotated[Optional[str], pydantic.Field(alias="quantityUom")] = None
    r"""The unit of measurement used for the goods"""

    gross_weight_kg: Annotated[
        Optional[float], pydantic.Field(alias="grossWeightKg")
    ] = None
    r"""The weight of the goods in kilograms"""

    gross_weight_lb: Annotated[
        Optional[float], pydantic.Field(alias="grossWeightLb")
    ] = None
    r"""The weight of the goods in pounds"""

    rate_class: Annotated[Optional[str], pydantic.Field(alias="rateClass")] = None
    r"""The rate class of the goods"""

    commodity_item_no: Annotated[
        Optional[str], pydantic.Field(alias="commodityItemNo")
    ] = None
    r"""The code of the goods"""

    chargeable_weight: Annotated[
        Optional[float], pydantic.Field(alias="chargeableWeight")
    ] = None
    r"""The chargeable weight of the goods"""

    rate_or_charge: Annotated[Optional[float], pydantic.Field(alias="rateOrCharge")] = (
        None
    )
    r"""Additional charges"""

    total: Optional[float] = None
    r"""The total amount of charges"""

    length: Optional[float] = None
    r"""The length of the package"""

    width: Optional[float] = None
    r"""The width of the package"""

    height: Optional[float] = None
    r"""The height of the package"""

    dimensions_uom: Annotated[Optional[str], pydantic.Field(alias="dimensionsUom")] = (
        None
    )
    r"""The unit of measurement used for the dimensions"""

    description: Optional[str] = None
    r"""A description of the goods"""


class DimensionsTypedDict(TypedDict):
    length: NotRequired[float]
    r"""The length of the package"""
    width: NotRequired[float]
    r"""The width of the package"""
    height: NotRequired[float]
    r"""The height of the package"""
    dimensions_uom: NotRequired[str]
    r"""The unit of measurement used for the dimensions"""


class Dimensions(BaseModel):
    length: Optional[float] = None
    r"""The length of the package"""

    width: Optional[float] = None
    r"""The width of the package"""

    height: Optional[float] = None
    r"""The height of the package"""

    dimensions_uom: Annotated[Optional[str], pydantic.Field(alias="dimensionsUom")] = (
        None
    )
    r"""The unit of measurement used for the dimensions"""


class TotalsTypedDict(TypedDict):
    total_quantity: NotRequired[float]
    r"""The total number of pieces"""
    total_quantity_uom: NotRequired[str]
    r"""The unit of measurement used for the goods"""
    total_weight: NotRequired[float]
    r"""The total weight of the goods"""
    total_weight_uom: NotRequired[str]
    r"""The unit of measurement used for the weight"""
    total_chargeable_weight: NotRequired[float]
    r"""The total chargeable weight of the goods"""
    rate_or_charge: NotRequired[float]
    r"""Additional charges"""
    total: NotRequired[float]
    r"""The total amount of charges"""
    volume: NotRequired[float]
    r"""The volume of the goods"""
    volume_uom: NotRequired[str]
    r"""The unit of measurement used for the volume"""


class Totals(BaseModel):
    total_quantity: Annotated[
        Optional[float], pydantic.Field(alias="totalQuantity")
    ] = None
    r"""The total number of pieces"""

    total_quantity_uom: Annotated[
        Optional[str], pydantic.Field(alias="totalQuantityUom")
    ] = None
    r"""The unit of measurement used for the goods"""

    total_weight: Annotated[Optional[float], pydantic.Field(alias="totalWeight")] = None
    r"""The total weight of the goods"""

    total_weight_uom: Annotated[
        Optional[str], pydantic.Field(alias="totalWeightUom")
    ] = None
    r"""The unit of measurement used for the weight"""

    total_chargeable_weight: Annotated[
        Optional[float], pydantic.Field(alias="totalChargeableWeight")
    ] = None
    r"""The total chargeable weight of the goods"""

    rate_or_charge: Annotated[Optional[float], pydantic.Field(alias="rateOrCharge")] = (
        None
    )
    r"""Additional charges"""

    total: Optional[float] = None
    r"""The total amount of charges"""

    volume: Optional[float] = None
    r"""The volume of the goods"""

    volume_uom: Annotated[Optional[str], pydantic.Field(alias="volumeUom")] = None
    r"""The unit of measurement used for the volume"""


class ChargesTypedDict(TypedDict):
    weight_charges: NotRequired[float]
    r"""The sum indicated in the air waybill exclusive of any additional charges"""
    valuation_charges: NotRequired[float]
    r"""Additional charges imposed on the shipper if the declared value of the cargo exceeds the carrier's insurance coverage"""
    tax: NotRequired[float]
    r"""The taxes as stated in the air waybill"""
    pickup_charge: NotRequired[float]
    r"""The pickup charge for the goods"""
    freight_advance: NotRequired[float]
    r"""The amount paid as advance freight"""
    total_prepaid: NotRequired[float]
    r"""The prepaid amount, including all the additional charges"""
    total_collect: NotRequired[float]
    r"""The amount payable upon receipt"""
    other_charges: NotRequired[List[float]]
    total_other_charges: NotRequired[float]
    r"""The total amount of additional charges"""


class Charges(BaseModel):
    weight_charges: Annotated[
        Optional[float], pydantic.Field(alias="weightCharges")
    ] = None
    r"""The sum indicated in the air waybill exclusive of any additional charges"""

    valuation_charges: Annotated[
        Optional[float], pydantic.Field(alias="valuationCharges")
    ] = None
    r"""Additional charges imposed on the shipper if the declared value of the cargo exceeds the carrier's insurance coverage"""

    tax: Optional[float] = None
    r"""The taxes as stated in the air waybill"""

    pickup_charge: Annotated[Optional[float], pydantic.Field(alias="pickupCharge")] = (
        None
    )
    r"""The pickup charge for the goods"""

    freight_advance: Annotated[
        Optional[float], pydantic.Field(alias="freightAdvance")
    ] = None
    r"""The amount paid as advance freight"""

    total_prepaid: Annotated[Optional[float], pydantic.Field(alias="totalPrepaid")] = (
        None
    )
    r"""The prepaid amount, including all the additional charges"""

    total_collect: Annotated[Optional[float], pydantic.Field(alias="totalCollect")] = (
        None
    )
    r"""The amount payable upon receipt"""

    other_charges: Annotated[
        Optional[List[float]], pydantic.Field(alias="otherCharges")
    ] = None

    total_other_charges: Annotated[
        Optional[float], pydantic.Field(alias="totalOtherCharges")
    ] = None
    r"""The total amount of additional charges"""


class AirWaybillFieldsTypedDict(TypedDict):
    r"""Fields extracted from an air waybill"""

    description: NotRequired[str]
    r"""A description of the goods"""
    commodity_item_no: NotRequired[str]
    r"""The code of the goods"""
    shipper: NotRequired[ShipperTypedDict]
    consignee: NotRequired[ConsigneeTypedDict]
    carriers_agent: NotRequired[CarriersAgentTypedDict]
    airline: NotRequired[AirlineTypedDict]
    air_waybill_number: NotRequired[AirWaybillNumberTypedDict]
    air_waybill_date: NotRequired[date]
    r"""The date of the air waybill"""
    airport_of_departure: NotRequired[AirportOfDepartureTypedDict]
    airport_of_destination: NotRequired[AirportOfDestinationTypedDict]
    flight_and_date: NotRequired[List[FlightAndDateTypedDict]]
    declared_value_for_carriage: NotRequired[float]
    r"""The value of the cargo declared to the carrier"""
    declared_value_for_customs: NotRequired[float]
    r"""The value of the cargo declared to the customs"""
    amount_of_insurance: NotRequired[float]
    r"""The amount of the insurance coverage on the goods"""
    currency: NotRequired[str]
    r"""The currency of the air waybill"""
    line_items: NotRequired[List[AirWaybillFieldsLineItemsTypedDict]]
    dimensions: NotRequired[List[DimensionsTypedDict]]
    totals: NotRequired[TotalsTypedDict]
    charges: NotRequired[ChargesTypedDict]


class AirWaybillFields(BaseModel):
    r"""Fields extracted from an air waybill"""

    description: Optional[str] = None
    r"""A description of the goods"""

    commodity_item_no: Annotated[
        Optional[str], pydantic.Field(alias="commodityItemNo")
    ] = None
    r"""The code of the goods"""

    shipper: Optional[Shipper] = None

    consignee: Optional[Consignee] = None

    carriers_agent: Annotated[
        Optional[CarriersAgent], pydantic.Field(alias="carriersAgent")
    ] = None

    airline: Optional[Airline] = None

    air_waybill_number: Annotated[
        Optional[AirWaybillNumber], pydantic.Field(alias="airWaybillNumber")
    ] = None

    air_waybill_date: Annotated[
        Optional[date], pydantic.Field(alias="airWaybillDate")
    ] = None
    r"""The date of the air waybill"""

    airport_of_departure: Annotated[
        Optional[AirportOfDeparture], pydantic.Field(alias="airportOfDeparture")
    ] = None

    airport_of_destination: Annotated[
        Optional[AirportOfDestination], pydantic.Field(alias="airportOfDestination")
    ] = None

    flight_and_date: Annotated[
        Optional[List[FlightAndDate]], pydantic.Field(alias="flightAndDate")
    ] = None

    declared_value_for_carriage: Annotated[
        Optional[float], pydantic.Field(alias="declaredValueForCarriage")
    ] = None
    r"""The value of the cargo declared to the carrier"""

    declared_value_for_customs: Annotated[
        Optional[float], pydantic.Field(alias="declaredValueForCustoms")
    ] = None
    r"""The value of the cargo declared to the customs"""

    amount_of_insurance: Annotated[
        Optional[float], pydantic.Field(alias="amountOfInsurance")
    ] = None
    r"""The amount of the insurance coverage on the goods"""

    currency: Optional[str] = None
    r"""The currency of the air waybill"""

    line_items: Annotated[
        Optional[List[AirWaybillFieldsLineItems]], pydantic.Field(alias="lineItems")
    ] = None

    dimensions: Optional[List[Dimensions]] = None

    totals: Optional[Totals] = None

    charges: Optional[Charges] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from abbyy_document_ai.types import BaseModel
from datetime import date
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ExporterTypedDict(TypedDict):
    r"""Information about the exporter/manufacturer of the goods"""

    name: NotRequired[str]
    r"""The name of the exporter/manufacturer of the goods"""
    address: NotRequired[str]
    r"""The address of the exporter/manufacturer of the goods"""


class Exporter(BaseModel):
    r"""Information about the exporter/manufacturer of the goods"""

    name: Optional[str] = None
    r"""The name of the exporter/manufacturer of the goods"""

    address: Optional[str] = None
    r"""The address of the exporter/manufacturer of the goods"""


class CertificateOfOriginFieldsConsigneeTypedDict(TypedDict):
    r"""Information about the importer/consignee of the goods"""

    name: NotRequired[str]
    r"""The name of the importer/consignee of the goods"""
    address: NotRequired[str]
    r"""The address of the importer/consignee of the goods"""


class CertificateOfOriginFieldsConsignee(BaseModel):
    r"""Information about the importer/consignee of the goods"""

    name: Optional[str] = None
    r"""The name of the importer/consignee of the goods"""

    address: Optional[str] = None
    r"""The address of the importer/consignee of the goods"""


class MeansOfTransportAndRouteTypedDict(TypedDict):
    city_of_origin: NotRequired[str]
    r"""The city of origin"""
    country_of_origin: NotRequired[str]
    r"""The country of origin"""
    city_of_destination: NotRequired[str]
    r"""The city of destination"""
    country_of_destination: NotRequired[str]
    r"""The country of destination"""
    means_of_transport: NotRequired[str]
    r"""The means of transport used to ship the goods"""
    departure_date: NotRequired[date]
    r"""The departure date of the transport"""


class MeansOfTransportAndRoute(BaseModel):
    city_of_origin: Annotated[Optional[str], pydantic.Field(alias="cityOfOrigin")] = (
        None
    )
    r"""The city of origin"""

    country_of_origin: Annotated[
        Optional[str], pydantic.Field(alias="countryOfOrigin")
    ] = None
    r"""The country of origin"""

    city_of_destination: Annotated[
        Optional[str], pydantic.Field(alias="cityOfDestination")
    ] = None
    r"""The city of destination"""

    country_of_destination: Annotated[
        Optional[str], pydantic.Field(alias="countryOfDestination")
    ] = None
    r"""The country of destination"""

    means_of_transport: Annotated[
        Optional[str], pydantic.Field(alias="meansOfTransport")
    ] = None
    r"""The means of transport used to ship the goods"""

    departure_date: Annotated[Optional[date], pydantic.Field(alias="departureDate")] = (
        None
    )
    r"""The departure date of the transport"""


class InvoiceNumbersTypedDict(TypedDict):
    invoice_number: NotRequired[str]
    r"""The invoice number"""
    invoice_date: NotRequired[date]
    r"""The invoice date"""


class InvoiceNumbers(BaseModel):
    invoice_number: Annotated[Optional[str], pydantic.Field(alias="invoiceNumber")] = (
        None
    )
    r"""The invoice number"""

    invoice_date: Annotated[Optional[date], pydantic.Field(alias="invoiceDate")] = None
    r"""The invoice date"""


class CertificateOfOriginFieldsLineItemsTypedDict(TypedDict):
    position: NotRequired[float]
    r"""The line item position number"""
    numbers_and_kind_of_packages: NotRequired[str]
    r"""A general description of the goods and their packaging"""
    description: NotRequired[str]
    r"""A general description of the goods and their packaging"""
    hs_code: NotRequired[str]
    r"""The Harmonized System (HS) code"""
    origin_criterion: NotRequired[str]
    r"""The Origin Criterion (WO, PE, or PSR)"""
    preference_criterion: NotRequired[str]
    r"""The Preference Criterion (A, B, C, D, E, or F)"""
    country_of_origin: NotRequired[str]
    r"""The country of origin of the goods"""
    quantity: NotRequired[float]
    r"""The number of units of goods"""
    uom: NotRequired[str]
    r"""The unit of measurement used for the goods"""
    net_weight: NotRequired[float]
    r"""The net weight of the goods"""
    gross_weight: NotRequired[float]
    r"""The gross weight of the goods"""
    weight_uom: NotRequired[str]
    r"""The unit of measurement for the weight of the goods"""


class CertificateOfOriginFieldsLineItems(BaseModel):
    position: Optional[float] = None
    r"""The line item position number"""

    numbers_and_kind_of_packages: Annotated[
        Optional[str], pydantic.Field(alias="numbersAndKindOfPackages")
    ] = None
    r"""A general description of the goods and their packaging"""

    description: Optional[str] = None
    r"""A general description of the goods and their packaging"""

    hs_code: Annotated[Optional[str], pydantic.Field(alias="hsCode")] = None
    r"""The Harmonized System (HS) code"""

    origin_criterion: Annotated[
        Optional[str], pydantic.Field(alias="originCriterion")
    ] = None
    r"""The Origin Criterion (WO, PE, or PSR)"""

    preference_criterion: Annotated[
        Optional[str], pydantic.Field(alias="preferenceCriterion")
    ] = None
    r"""The Preference Criterion (A, B, C, D, E, or F)"""

    country_of_origin: Annotated[
        Optional[str], pydantic.Field(alias="countryOfOrigin")
    ] = None
    r"""The country of origin of the goods"""

    quantity: Optional[float] = None
    r"""The number of units of goods"""

    uom: Optional[str] = None
    r"""The unit of measurement used for the goods"""

    net_weight: Annotated[Optional[float], pydantic.Field(alias="netWeight")] = None
    r"""The net weight of the goods"""

    gross_weight: Annotated[Optional[float], pydantic.Field(alias="grossWeight")] = None
    r"""The gross weight of the goods"""

    weight_uom: Annotated[Optional[str], pydantic.Field(alias="weightUom")] = None
    r"""The unit of measurement for the weight of the goods"""


class CertificateOfOriginFieldsTypedDict(TypedDict):
    r"""Fields extracted from a certificate of origin"""

    certificate_number: NotRequired[str]
    r"""The certificate number"""
    serial_number: NotRequired[str]
    r"""The certificate serial number"""
    country_of_issue: NotRequired[str]
    r"""The certificate country of issue"""
    exporter: NotRequired[ExporterTypedDict]
    r"""Information about the exporter/manufacturer of the goods"""
    consignee: NotRequired[CertificateOfOriginFieldsConsigneeTypedDict]
    r"""Information about the importer/consignee of the goods"""
    means_of_transport_and_route: NotRequired[MeansOfTransportAndRouteTypedDict]
    invoice_numbers: NotRequired[List[InvoiceNumbersTypedDict]]
    marks_and_numbers_of_packages: NotRequired[str]
    r"""Package marking and numbering"""
    line_items: NotRequired[List[CertificateOfOriginFieldsLineItemsTypedDict]]
    weight: NotRequired[float]
    r"""The total weight of the goods"""
    weight_uom: NotRequired[str]
    r"""The unit of measurement for the total weight of the goods"""


class CertificateOfOriginFields(BaseModel):
    r"""Fields extracted from a certificate of origin"""

    certificate_number: Annotated[
        Optional[str], pydantic.Field(alias="certificateNumber")
    ] = None
    r"""The certificate number"""

    serial_number: Annotated[Optional[str], pydantic.Field(alias="serialNumber")] = None
    r"""The certificate serial number"""

    country_of_issue: Annotated[
        Optional[str], pydantic.Field(alias="countryOfIssue")
    ] = None
    r"""The certificate country of issue"""

    exporter: Optional[Exporter] = None
    r"""Information about the exporter/manufacturer of the goods"""

    consignee: Optional[CertificateOfOriginFieldsConsignee] = None
    r"""Information about the importer/consignee of the goods"""

    means_of_transport_and_route: Annotated[
        Optional[MeansOfTransportAndRoute],
        pydantic.Field(alias="meansOfTransportAndRoute"),
    ] = None

    invoice_numbers: Annotated[
        Optional[List[InvoiceNumbers]], pydantic.Field(alias="invoiceNumbers")
    ] = None

    marks_and_numbers_of_packages: Annotated[
        Optional[str], pydantic.Field(alias="marksAndNumbersOfPackages")
    ] = None
    r"""Package marking and numbering"""

    line_items: Annotated[
        Optional[List[CertificateOfOriginFieldsLineItems]],
        pydantic.Field(alias="lineItems"),
    ] = None

    weight: Optional[float] = None
    r"""The total weight of the goods"""

    weight_uom: Annotated[Optional[str], pydantic.Field(alias="weightUom")] = None
    r"""The unit of measurement for the total weight of the goods"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .document import Document, DocumentTypedDict
from abbyy_document_ai.types import BaseModel
from enum import Enum
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class Rotated(str, Enum):
    r"""Rotation of the page relative to the original image."""

    NONE = "none"
    CLOCKWISE = "clockwise"
    COUNTERCLOCKWISE = "counterclockwise"
    UPSIDE_DOWN = "upside-down"


class PositionTypedDict(TypedDict):
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    l: NotRequired[int]
    r"""Left coordinate."""
    t: NotRequired[int]
    r"""Top coordinate."""
    r: NotRequired[int]
    r"""Right coordinate."""
    b: NotRequired[int]
    r"""Bottom coordinate."""


class Position(BaseModel):
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    l: Optional[int] = None
    r"""Left coordinate."""

    t: Optional[int] = None
    r"""Top coordinate."""

    r: Optional[int] = None
    r"""Right coordinate."""

    b: Optional[int] = None
    r"""Bottom coordinate."""


class ExtractedTextTextLayoutPagesTextsPositionTypedDict(TypedDict):
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    l: NotRequired[int]
    r"""Left coordinate."""
    t: NotRequired[int]
    r"""Top coordinate."""
    r: NotRequired[int]
    r"""Right coordinate."""
    b: NotRequired[int]
    r"""Bottom coordinate."""


class ExtractedTextTextLayoutPagesTextsPosition(BaseModel):
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    l: Optional[int] = None
    r"""Left coordinate."""

    t: Optional[int] = None
    r"""Top coordinate."""

    r: Optional[int] = None
    r"""Right coordinate."""

    b: Optional[int] = None
    r"""Bottom coordinate."""


class CharParamsTypedDict(TypedDict):
    r"""Symbol font formatting parameters."""

    bold: NotRequired[bool]
    italic: NotRequired[bool]
    underlined: NotRequired[bool]
    strikeout: NotRequired[bool]
    small_caps: NotRequired[bool]
    superscript: NotRequired[bool]
    subscript: NotRequired[bool]
    scaling: NotRequired[int]
    spacing: NotRequired[int]
    font_size: NotRequired[int]
    font_name: NotRequired[str]
    color: NotRequired[str]
    lang: NotRequired[str]


class CharParams(BaseModel):
    r"""Symbol font formatting parameters."""

    bold: Optional[bool] = None

    italic: Optional[bool] = None

    underlined: Optional[bool] = None

    strikeout: Optional[bool] = None

    small_caps: Annotated[Optional[bool], pydantic.Field(alias="smallCaps")] = None

    superscript: Optional[bool] = None

    subscript: Optional[bool] = None

    scaling: Optional[int] = None

    spacing: Optional[int] = None

    font_size: Annotated[Optional[int], pydantic.Field(alias="fontSize")] = None

    font_name: Annotated[Optional[str], pydantic.Field(alias="fontName")] = None

    color: Optional[str] = None

    lang: Optional[str] = None


class ExtractedTextTextLayoutPagesTextsLinesPositionTypedDict(TypedDict):
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    l: NotRequired[int]
    r"""Left coordinate."""
    t: NotRequired[int]
    r"""Top coordinate."""
    r: NotRequired[int]
    r"""Right coordinate."""
    b: NotRequired[int]
    r"""Bottom coordinate."""


class ExtractedTextTextLayoutPagesTextsLinesPosition(BaseModel):
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    l: Optional[int] = None
    r"""Left coordinate."""

    t: Optional[int] = None
    r"""Top coordinate."""

    r: Optional[int] = None
    r"""Right coordinate."""

    b: Optional[int] = None
    r"""Bottom coordinate."""


class ExtractedTextTextLayoutPagesTextsLinesWordsPositionTypedDict(TypedDict):
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    l: NotRequired[int]
    r"""Left coordinate."""
    t: NotRequired[int]
    r"""Top coordinate."""
    r: NotRequired[int]
    r"""Right coordinate."""
    b: NotRequired[int]
    r"""Bottom coordinate."""


class ExtractedTextTextLayoutPagesTextsLinesWordsPosition(BaseModel):
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    l: Optional[int] = None
    r"""Left coordinate."""

    t: Optional[int] = None
    r"""Top coordinate."""

    r: Optional[int] = None
    r"""Right coordinate."""

    b: Optional[int] = None
    r"""Bottom coordinate."""


class CharsTypedDict(TypedDict):
    confidence: NotRequired[float]
    r"""Confidence."""
    text: NotRequired[str]
    position: NotRequired[ExtractedTextTextLayoutPagesTextsLinesWordsPositionTypedDict]
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""


class Chars(BaseModel):
    confidence: Optional[float] = None
    r"""Confidence."""

    text: Optional[str] = None

    position: Optional[ExtractedTextTextLayoutPagesTextsLinesWordsPosition] = None
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""


class WordsTypedDict(TypedDict):
    r"""An array of words in a line of text ordered by reading direction."""

    confidence: NotRequired[float]
    r"""Confidence."""
    position: NotRequired[ExtractedTextTextLayoutPagesTextsLinesPositionTypedDict]
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""
    text: NotRequired[str]
    chars: NotRequired[List[CharsTypedDict]]


class Words(BaseModel):
    r"""An array of words in a line of text ordered by reading direction."""

    confidence: Optional[float] = None
    r"""Confidence."""

    position: Optional[ExtractedTextTextLayoutPagesTextsLinesPosition] = None
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    text: Optional[str] = None

    chars: Optional[List[Chars]] = None


class LinesTypedDict(TypedDict):
    r"""A line of text."""

    confidence: NotRequired[float]
    r"""Confidence."""
    position: NotRequired[ExtractedTextTextLayoutPagesTextsPositionTypedDict]
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""
    text: NotRequired[str]
    r"""A line of text value."""
    char_params: NotRequired[CharParamsTypedDict]
    r"""Symbol font formatting parameters."""
    words: NotRequired[List[WordsTypedDict]]


class Lines(BaseModel):
    r"""A line of text."""

    confidence: Optional[float] = None
    r"""Confidence."""

    position: Optional[ExtractedTextTextLayoutPagesTextsPosition] = None
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    text: Optional[str] = None
    r"""A line of text value."""

    char_params: Annotated[Optional[CharParams], pydantic.Field(alias="charParams")] = (
        None
    )
    r"""Symbol font formatting parameters."""

    words: Optional[List[Words]] = None


class TextsTypedDict(TypedDict):
    r"""Text."""

    id: NotRequired[str]
    r"""Text block ID."""
    position: NotRequired[PositionTypedDict]
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""
    lines: NotRequired[List[LinesTypedDict]]
    r"""An array of text strings in the text block ordered by reading direction."""
    confidence: NotRequired[float]
    r"""Confidence."""


class Texts(BaseModel):
    r"""Text."""

    id: Optional[str] = None
    r"""Text block ID."""

    position: Optional[Position] = None
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    lines: Optional[List[Lines]] = None
    r"""An array of text strings in the text block ordered by reading direction."""

    confidence: Optional[float] = None
    r"""Confidence."""


class ExtractedTextPositionTypedDict(TypedDict):
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    l: NotRequired[int]
    r"""Left coordinate."""
    t: NotRequired[int]
    r"""Top coordinate."""
    r: NotRequired[int]
    r"""Right coordinate."""
    b: NotRequired[int]
    r"""Bottom coordinate."""


class ExtractedTextPosition(BaseModel):
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    l: Optional[int] = None
    r"""Left coordinate."""

    t: Optional[int] = None
    r"""Top coordinate."""

    r: Optional[int] = None
    r"""Right coordinate."""

    b: Optional[int] = None
    r"""Bottom coordinate."""


class ExtractedTextTextLayoutPagesTablesPositionTypedDict(TypedDict):
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    l: NotRequired[int]
    r"""Left coordinate."""
    t: NotRequired[int]
    r"""Top coordinate."""
    r: NotRequired[int]
    r"""Right coordinate."""
    b: NotRequired[int]
    r"""Bottom coordinate."""


class ExtractedTextTextLayoutPagesTablesPosition(BaseModel):
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    l: Optional[int] = None
    r"""Left coordinate."""

    t: Optional[int] = None
    r"""Top coordinate."""

    r: Optional[int] = None
    r"""Right coordinate."""

    b: Optional[int] = None
    r"""Bottom coordinate."""


class ColRowPositionTypedDict(TypedDict):
    r"""Cell coordinate in the Column-Row system {l:leftColumn, t:topRow, r:rightColumn, b:bottomRow}"""

    l: int
    r"""Left separator index."""
    t: int
    r"""Top separator index."""
    r: int
    r"""Right separator index."""
    b: int
    r"""Bottom separator index."""


class ColRowPosition(BaseModel):
    r"""Cell coordinate in the Column-Row system {l:leftColumn, t:topRow, r:rightColumn, b:bottomRow}"""

    l: int
    r"""Left separator index."""

    t: int
    r"""Top separator index."""

    r: int
    r"""Right separator index."""

    b: int
    r"""Bottom separator index."""


class L(str, Enum):
    UNKNOWN = "unknown"
    INVISIBLE = "invisible"
    VISIBLE = "visible"


class T(str, Enum):
    UNKNOWN = "unknown"
    INVISIBLE = "invisible"
    VISIBLE = "visible"


class R(str, Enum):
    UNKNOWN = "unknown"
    INVISIBLE = "invisible"
    VISIBLE = "visible"


class B(str, Enum):
    UNKNOWN = "unknown"
    INVISIBLE = "invisible"
    VISIBLE = "visible"


class BordersTypedDict(TypedDict):
    r"""Table cell border types {l:left, t:top, r:right, b:bottom}"""

    l: NotRequired[L]
    t: NotRequired[T]
    r: NotRequired[R]
    b: NotRequired[B]


class Borders(BaseModel):
    r"""Table cell border types {l:left, t:top, r:right, b:bottom}"""

    l: Optional[L] = None

    t: Optional[T] = None

    r: Optional[R] = None

    b: Optional[B] = None


class ContentType(str, Enum):
    r"""Table cell content type: [text, picture, barcode]."""

    TEXT = "text"
    PICTURE = "picture"
    BARCODE = "barcode"


class ExtractedTextTextLayoutPagesTablesCellsPositionTypedDict(TypedDict):
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    l: NotRequired[int]
    r"""Left coordinate."""
    t: NotRequired[int]
    r"""Top coordinate."""
    r: NotRequired[int]
    r"""Right coordinate."""
    b: NotRequired[int]
    r"""Bottom coordinate."""


class ExtractedTextTextLayoutPagesTablesCellsPosition(BaseModel):
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    l: Optional[int] = None
    r"""Left coordinate."""

    t: Optional[int] = None
    r"""Top coordinate."""

    r: Optional[int] = None
    r"""Right coordinate."""

    b: Optional[int] = None
    r"""Bottom coordinate."""


class ExtractedTextCharParamsTypedDict(TypedDict):
    r"""Symbol font formatting parameters."""

    bold: NotRequired[bool]
    italic: NotRequired[bool]
    underlined: NotRequired[bool]
    strikeout: NotRequired[bool]
    small_caps: NotRequired[bool]
    superscript: NotRequired[bool]
    subscript: NotRequired[bool]
    scaling: NotRequired[int]
    spacing: NotRequired[int]
    font_size: NotRequired[int]
    font_name: NotRequired[str]
    color: NotRequired[str]
    lang: NotRequired[str]


class ExtractedTextCharParams(BaseModel):
    r"""Symbol font formatting parameters."""

    bold: Optional[bool] = None

    italic: Optional[bool] = None

    underlined: Optional[bool] = None

    strikeout: Optional[bool] = None

    small_caps: Annotated[Optional[bool], pydantic.Field(alias="smallCaps")] = None

    superscript: Optional[bool] = None

    subscript: Optional[bool] = None

    scaling: Optional[int] = None

    spacing: Optional[int] = None

    font_size: Annotated[Optional[int], pydantic.Field(alias="fontSize")] = None

    font_name: Annotated[Optional[str], pydantic.Field(alias="fontName")] = None

    color: Optional[str] = None

    lang: Optional[str] = None


class ExtractedTextTextLayoutPagesTablesCellsLinesPositionTypedDict(TypedDict):
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    l: NotRequired[int]
    r"""Left coordinate."""
    t: NotRequired[int]
    r"""Top coordinate."""
    r: NotRequired[int]
    r"""Right coordinate."""
    b: NotRequired[int]
    r"""Bottom coordinate."""


class ExtractedTextTextLayoutPagesTablesCellsLinesPosition(BaseModel):
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    l: Optional[int] = None
    r"""Left coordinate."""

    t: Optional[int] = None
    r"""Top coordinate."""

    r: Optional[int] = None
    r"""Right coordinate."""

    b: Optional[int] = None
    r"""Bottom coordinate."""


class ExtractedTextTextLayoutPagesTablesCellsLinesWordsPositionTypedDict(TypedDict):
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    l: NotRequired[int]
    r"""Left coordinate."""
    t: NotRequired[int]
    r"""Top coordinate."""
    r: NotRequired[int]
    r"""Right coordinate."""
    b: NotRequired[int]
    r"""Bottom coordinate."""


class ExtractedTextTextLayoutPagesTablesCellsLinesWordsPosition(BaseModel):
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    l: Optional[int] = None
    r"""Left coordinate."""

    t: Optional[int] = None
    r"""Top coordinate."""

    r: Optional[int] = None
    r"""Right coordinate."""

    b: Optional[int] = None
    r"""Bottom coordinate."""


class ExtractedTextCharsTypedDict(TypedDict):
    confidence: NotRequired[float]
    r"""Confidence."""
    text: NotRequired[str]
    position: NotRequired[
        ExtractedTextTextLayoutPagesTablesCellsLinesWordsPositionTypedDict
    ]
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""


class ExtractedTextChars(BaseModel):
    confidence: Optional[float] = None
    r"""Confidence."""

    text: Optional[str] = None

    position: Optional[ExtractedTextTextLayoutPagesTablesCellsLinesWordsPosition] = None
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""


class ExtractedTextWordsTypedDict(TypedDict):
    r"""An array of words in a line of text ordered by reading direction."""

    confidence: NotRequired[float]
    r"""Confidence."""
    position: NotRequired[ExtractedTextTextLayoutPagesTablesCellsLinesPositionTypedDict]
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""
    text: NotRequired[str]
    chars: NotRequired[List[ExtractedTextCharsTypedDict]]


class ExtractedTextWords(BaseModel):
    r"""An array of words in a line of text ordered by reading direction."""

    confidence: Optional[float] = None
    r"""Confidence."""

    position: Optional[ExtractedTextTextLayoutPagesTablesCellsLinesPosition] = None
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    text: Optional[str] = None

    chars: Optional[List[ExtractedTextChars]] = None


class ExtractedTextLinesTypedDict(TypedDict):
    r"""A line of text."""

    confidence: NotRequired[float]
    r"""Confidence."""
    position: NotRequired[ExtractedTextTextLayoutPagesTablesCellsPositionTypedDict]
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""
    text: NotRequired[str]
    r"""A line of text value."""
    char_params: NotRequired[ExtractedTextCharParamsTypedDict]
    r"""Symbol font formatting parameters."""
    words: NotRequired[List[ExtractedTextWordsTypedDict]]


class ExtractedTextLines(BaseModel):
    r"""A line of text."""

    confidence: Optional[float] = None
    r"""Confidence."""

    position: Optional[ExtractedTextTextLayoutPagesTablesCellsPosition] = None
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    text: Optional[str] = None
    r"""A line of text value."""

    char_params: Annotated[
        Optional[ExtractedTextCharParams], pydantic.Field(alias="charParams")
    ] = None
    r"""Symbol font formatting parameters."""

    words: Optional[List[ExtractedTextWords]] = None


class CellsTypedDict(TypedDict):
    r"""Table cell."""

    id: NotRequired[str]
    r"""Table cell ID."""
    position: NotRequired[ExtractedTextTextLayoutPagesTablesPositionTypedDict]
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""
    confidence: NotRequired[float]
    r"""Confidence."""
    col_row_position: NotRequired[ColRowPositionTypedDict]
    r"""Cell coordinate in the Column-Row system {l:leftColumn, t:topRow, r:rightColumn, b:bottomRow}"""
    borders: NotRequired[BordersTypedDict]
    r"""Table cell border types {l:left, t:top, r:right, b:bottom}"""
    content_type: NotRequired[ContentType]
    r"""Table cell content type: [text, picture, barcode]."""
    lines: NotRequired[List[ExtractedTextLinesTypedDict]]
    r"""An array of text strings in the text block ordered by reading direction."""


class Cells(BaseModel):
    r"""Table cell."""

    id: Optional[str] = None
    r"""Table cell ID."""

    position: Optional[ExtractedTextTextLayoutPagesTablesPosition] = None
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    confidence: Optional[float] = None
    r"""Confidence."""

    col_row_position: Annotated[
        Optional[ColRowPosition], pydantic.Field(alias="colRowPosition")
    ] = None
    r"""Cell coordinate in the Column-Row system {l:leftColumn, t:topRow, r:rightColumn, b:bottomRow}"""

    borders: Optional[Borders] = None
    r"""Table cell border types {l:left, t:top, r:right, b:bottom}"""

    content_type: Annotated[
        Optional[ContentType], pydantic.Field(alias="contentType")
    ] = None
    r"""Table cell content type: [text, picture, barcode]."""

    lines: Optional[List[ExtractedTextLines]] = None
    r"""An array of text strings in the text block ordered by reading direction."""


class TablesTypedDict(TypedDict):
    r"""Table."""

    id: NotRequired[str]
    r"""Table block ID."""
    confidence: NotRequired[float]
    r"""Confidence."""
    position: NotRequired[ExtractedTextPositionTypedDict]
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""
    cells: NotRequired[List[CellsTypedDict]]
    r"""Table cells."""


class Tables(BaseModel):
    r"""Table."""

    id: Optional[str] = None
    r"""Table block ID."""

    confidence: Optional[float] = None
    r"""Confidence."""

    position: Optional[ExtractedTextPosition] = None
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    cells: Optional[List[Cells]] = None
    r"""Table cells."""


class ExtractedTextTextPositionTypedDict(TypedDict):
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    l: NotRequired[int]
    r"""Left coordinate."""
    t: NotRequired[int]
    r"""Top coordinate."""
    r: NotRequired[int]
    r"""Right coordinate."""
    b: NotRequired[int]
    r"""Bottom coordinate."""


class ExtractedTextTextPosition(BaseModel):
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    l: Optional[int] = None
    r"""Left coordinate."""

    t: Optional[int] = None
    r"""Top coordinate."""

    r: Optional[int] = None
    r"""Right coordinate."""

    b: Optional[int] = None
    r"""Bottom coordinate."""


class Value(str, Enum):
    CHECKED = "checked"
    UNCHECKED = "unchecked"
    CORRECTED = "corrected"
    UNKNOWN = "unknown"


class CheckmarksTypedDict(TypedDict):
    position: NotRequired[ExtractedTextTextPositionTypedDict]
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""
    confidence: NotRequired[float]
    r"""Confidence."""
    value: NotRequired[Value]


class Checkmarks(BaseModel):
    position: Optional[ExtractedTextTextPosition] = None
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    confidence: Optional[float] = None
    r"""Confidence."""

    value: Optional[Value] = None


class ExtractedTextTextLayoutPositionTypedDict(TypedDict):
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    l: NotRequired[int]
    r"""Left coordinate."""
    t: NotRequired[int]
    r"""Top coordinate."""
    r: NotRequired[int]
    r"""Right coordinate."""
    b: NotRequired[int]
    r"""Bottom coordinate."""


class ExtractedTextTextLayoutPosition(BaseModel):
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    l: Optional[int] = None
    r"""Left coordinate."""

    t: Optional[int] = None
    r"""Top coordinate."""

    r: Optional[int] = None
    r"""Right coordinate."""

    b: Optional[int] = None
    r"""Bottom coordinate."""


class ExtractedTextTextType(str, Enum):
    r"""Barcode type."""

    CODE39 = "Code39"
    INTERLEAVED25 = "Interleaved25"
    EAN13 = "EAN13"
    CODE128 = "Code128"
    EAN8 = "EAN8"
    PDF417 = "PDF417"
    CODABAR = "Codabar"
    UPCE = "UPCE"
    INDUSTRIAL25 = "Industrial25"
    IATA25 = "IATA25"
    MATRIX25 = "Matrix25"
    CODE93 = "Code93"
    POST_NET = "PostNet"
    UCC128 = "UCC128"
    PATCH = "Patch"
    AZTEC = "Aztec"
    DATA_MATRIX = "DataMatrix"
    QR_CODE = "QRCode"
    UPCA = "UPCA"
    MAXI_CODE = "MaxiCode"
    CODE32 = "Code32"
    FULL_ASCII = "FullAscii"
    INTELLIGENT_MAIL = "IntelligentMail"
    ROYAL_MAIL4_STATE = "RoyalMail4State"
    KIX = "KIX"
    AUSTRALIA4_STATE = "Australia4State"
    JAPAN_POST = "JapanPost"
    NOT_FOUND = "NotFound"


class SupplementType(str, Enum):
    r"""Supplement type."""

    NONE = "none"
    TWODIGITS = "2digits"
    FIVEDIGITS = "5digits"


class BarcodesTypedDict(TypedDict):
    r"""Barcode."""

    id: NotRequired[str]
    r"""Barcode block ID."""
    confidence: NotRequired[float]
    r"""Confidence."""
    position: NotRequired[ExtractedTextTextLayoutPositionTypedDict]
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""
    type: NotRequired[ExtractedTextTextType]
    r"""Barcode type."""
    value: NotRequired[str]
    r"""Barcode value."""
    supplement_value: NotRequired[str]
    r"""Supplement value."""
    supplement_type: NotRequired[SupplementType]
    r"""Supplement type."""


class Barcodes(BaseModel):
    r"""Barcode."""

    id: Optional[str] = None
    r"""Barcode block ID."""

    confidence: Optional[float] = None
    r"""Confidence."""

    position: Optional[ExtractedTextTextLayoutPosition] = None
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    type: Optional[ExtractedTextTextType] = None
    r"""Barcode type."""

    value: Optional[str] = None
    r"""Barcode value."""

    supplement_value: Annotated[
        Optional[str], pydantic.Field(alias="supplementValue")
    ] = None
    r"""Supplement value."""

    supplement_type: Annotated[
        Optional[SupplementType], pydantic.Field(alias="supplementType")
    ] = None
    r"""Supplement type."""


class ExtractedTextTextLayoutPagesPositionTypedDict(TypedDict):
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    l: NotRequired[int]
    r"""Left coordinate."""
    t: NotRequired[int]
    r"""Top coordinate."""
    r: NotRequired[int]
    r"""Right coordinate."""
    b: NotRequired[int]
    r"""Bottom coordinate."""


class ExtractedTextTextLayoutPagesPosition(BaseModel):
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    l: Optional[int] = None
    r"""Left coordinate."""

    t: Optional[int] = None
    r"""Top coordinate."""

    r: Optional[int] = None
    r"""Right coordinate."""

    b: Optional[int] = None
    r"""Bottom coordinate."""


class PicturesTypedDict(TypedDict):
    r"""Block with image."""

    confidence: NotRequired[float]
    r"""Confidence."""
    id: NotRequired[str]
    r"""Picture block ID."""
    position: NotRequired[ExtractedTextTextLayoutPagesPositionTypedDict]
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""


class Pictures(BaseModel):
    r"""Block with image."""

    confidence: Optional[float] = None
    r"""Confidence."""

    id: Optional[str] = None
    r"""Picture block ID."""

    position: Optional[ExtractedTextTextLayoutPagesPosition] = None
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""


class ExtractedTextTextLayoutPagesSeparatorsPositionTypedDict(TypedDict):
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    l: NotRequired[int]
    r"""Left coordinate."""
    t: NotRequired[int]
    r"""Top coordinate."""
    r: NotRequired[int]
    r"""Right coordinate."""
    b: NotRequired[int]
    r"""Bottom coordinate."""


class ExtractedTextTextLayoutPagesSeparatorsPosition(BaseModel):
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    l: Optional[int] = None
    r"""Left coordinate."""

    t: Optional[int] = None
    r"""Top coordinate."""

    r: Optional[int] = None
    r"""Right coordinate."""

    b: Optional[int] = None
    r"""Bottom coordinate."""


class ExtractedTextType(str, Enum):
    r"""Separator type."""

    SOLID = "solid"
    UNKNOWN = "unknown"
    DOTTED = "dotted"


class EndPointsTypedDict(TypedDict):
    start_x: NotRequired[int]
    r"""Start point X-coordinate."""
    start_y: NotRequired[int]
    r"""Start point Y-coordinate."""
    end_x: NotRequired[int]
    r"""End point X-coordinate."""
    end_y: NotRequired[int]
    r"""End point Y-coordinate."""


class EndPoints(BaseModel):
    start_x: Annotated[Optional[int], pydantic.Field(alias="startX")] = None
    r"""Start point X-coordinate."""

    start_y: Annotated[Optional[int], pydantic.Field(alias="startY")] = None
    r"""Start point Y-coordinate."""

    end_x: Annotated[Optional[int], pydantic.Field(alias="endX")] = None
    r"""End point X-coordinate."""

    end_y: Annotated[Optional[int], pydantic.Field(alias="endY")] = None
    r"""End point Y-coordinate."""


class SeparatorsTypedDict(TypedDict):
    r"""Separator."""

    confidence: NotRequired[float]
    r"""Confidence."""
    position: NotRequired[ExtractedTextTextLayoutPagesSeparatorsPositionTypedDict]
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""
    type: NotRequired[ExtractedTextType]
    r"""Separator type."""
    color: NotRequired[int]
    r"""Separator color."""
    thickness: NotRequired[int]
    r"""Separator thickness."""
    end_points: NotRequired[EndPointsTypedDict]


class Separators(BaseModel):
    r"""Separator."""

    confidence: Optional[float] = None
    r"""Confidence."""

    position: Optional[ExtractedTextTextLayoutPagesSeparatorsPosition] = None
    r"""Rectangle {r:left, t:top, r:right, b:bottom}"""

    type: Optional[ExtractedTextType] = None
    r"""Separator type."""

    color: Optional[int] = None
    r"""Separator color."""

    thickness: Optional[int] = None
    r"""Separator thickness."""

    end_points: Annotated[Optional[EndPoints], pydantic.Field(alias="endPoints")] = None


class PagesTypedDict(TypedDict):
    r"""Document page."""

    width: NotRequired[float]
    r"""The width of the page in pixels."""
    height: NotRequired[float]
    r"""The height of the page in points."""
    rotated: NotRequired[Rotated]
    r"""Rotation of the page relative to the original image."""
    texts: NotRequired[List[TextsTypedDict]]
    r"""An array of blocks with text."""
    tables: NotRequired[List[TablesTypedDict]]
    r"""An array of blocks with a table."""
    checkmarks: NotRequired[List[CheckmarksTypedDict]]
    barcodes: NotRequired[List[BarcodesTypedDict]]
    r"""Array of blocks with barcode."""
    pictures: NotRequired[List[PicturesTypedDict]]
    r"""Array of blocks with a picture."""
    separators: NotRequired[List[SeparatorsTypedDict]]
    r"""Array of blocks with separator."""


class Pages(BaseModel):
    r"""Document page."""

    width: Optional[float] = None
    r"""The width of the page in pixels."""

    height: Optional[float] = None
    r"""The height of the page in points."""

    rotated: Optional[Rotated] = None
    r"""Rotation of the page relative to the original image."""

    texts: Optional[List[Texts]] = None
    r"""An array of blocks with text."""

    tables: Optional[List[Tables]] = None
    r"""An array of blocks with a table."""

    checkmarks: Optional[List[Checkmarks]] = None

    barcodes: Optional[List[Barcodes]] = None
    r"""Array of blocks with barcode."""

    pictures: Optional[List[Pictures]] = None
    r"""Array of blocks with a picture."""

    separators: Optional[List[Separators]] = None
    r"""Array of blocks with separator."""


class LayoutTypedDict(TypedDict):
    r"""Layout (physical structure)."""

    pages: List[PagesTypedDict]
    r"""Array of document pages ordered from first to last."""
    corrected: NotRequired[bool]
    r"""Coordinates for the corrected image on the output."""


class Layout(BaseModel):
    r"""Layout (physical structure)."""

    pages: List[Pages]
    r"""Array of document pages ordered from first to last."""

    corrected: Optional[bool] = None
    r"""Coordinates for the corrected image on the output."""


class Role(str, Enum):
    r"""Paragraph role in the document."""

    OTHER = "other"
    TEXT = "text"
    HEADING = "heading"
    HEADING_NUMBER = "headingNumber"
    TABLE_OF_CONTENTS = "tableOfContents"
    TABLE_TEXT = "tableText"
    RUNNING_TITLE = "runningTitle"
    END_NOTE = "endNote"
    FOOT_NOTE = "footNote"
    TABLE_CAPTION = "tableCaption"
    TABLE_HEADING = "tableHeading"
    PICTURE_CAPTION = "pictureCaption"
    ARTEFACT = "artefact"


class Aligning(str, Enum):
    r"""Text aligning in the paragraph."""

    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"
    JUSTIFIED = "justified"
    JUSTIFIED_FOR_ARABIC = "justifiedForArabic"


class FormattingTypedDict(TypedDict):
    r"""Paragraph formatting."""

    aligning: NotRequired[Aligning]
    r"""Text aligning in the paragraph."""
    line_spacing: NotRequired[int]
    r"""Paragraph line spacing."""


class Formatting(BaseModel):
    r"""Paragraph formatting."""

    aligning: Optional[Aligning] = None
    r"""Text aligning in the paragraph."""

    line_spacing: Annotated[Optional[int], pydantic.Field(alias="lineSpacing")] = 0
    r"""Paragraph line spacing."""


class BlockType(str, Enum):
    r"""Paragraph container type."""

    TEXT = "text"
    CELL = "cell"


class LayoutReferencesTypedDict(TypedDict):
    r"""Reference block/cell id, paragraph index and line range in the block/cell."""

    block_id: str
    r"""Containing text block or table cell ID."""
    block_type: BlockType
    r"""Paragraph container type."""
    section_index: int
    r"""Index of logical section on page (or -1 if there is no section containing the paragraph - like headers, footers etc.)."""
    column_index: int
    r"""Index of column in logical section (or -1 if there is no column containing the paragraph - like incuts, footnotes etc)."""
    line_numbering: bool
    r"""true if the paragraph is contained in line numbering area."""
    par_index: int
    r"""Paragraph index in block or table cell."""
    first_line: int
    r"""Index of the first line of the paragraph in block or table cell."""
    last_line: int
    r"""Index of the last line of the paragraph in block or table cell."""


class LayoutReferences(BaseModel):
    r"""Reference block/cell id, paragraph index and line range in the block/cell."""

    block_id: Annotated[str, pydantic.Field(alias="blockId")]
    r"""Containing text block or table cell ID."""

    block_type: Annotated[BlockType, pydantic.Field(alias="blockType")]
    r"""Paragraph container type."""

    section_index: Annotated[int, pydantic.Field(alias="sectionIndex")]
    r"""Index of logical section on page (or -1 if there is no section containing the paragraph - like headers, footers etc.)."""

    column_index: Annotated[int, pydantic.Field(alias="columnIndex")]
    r"""Index of column in logical section (or -1 if there is no column containing the paragraph - like incuts, footnotes etc)."""

    line_numbering: Annotated[bool, pydantic.Field(alias="lineNumbering")]
    r"""true if the paragraph is contained in line numbering area."""

    par_index: Annotated[int, pydantic.Field(alias="parIndex")]
    r"""Paragraph index in block or table cell."""

    first_line: Annotated[int, pydantic.Field(alias="firstLine")]
    r"""Index of the first line of the paragraph in block or table cell."""

    last_line: Annotated[int, pydantic.Field(alias="lastLine")]
    r"""Index of the last line of the paragraph in block or table cell."""


class ListReferenceTypedDict(TypedDict):
    r"""Reference to list containing the paragraph."""

    id: NotRequired[str]
    r"""List ID."""
    level_index: NotRequired[int]
    r"""Current list item level index."""
    ordinal_number: NotRequired[int]
    r"""Current list item ordinal number."""


class ListReference(BaseModel):
    r"""Reference to list containing the paragraph."""

    id: Optional[str] = None
    r"""List ID."""

    level_index: Annotated[Optional[int], pydantic.Field(alias="levelIndex")] = 0
    r"""Current list item level index."""

    ordinal_number: Annotated[Optional[int], pydantic.Field(alias="ordinalNumber")] = 0
    r"""Current list item ordinal number."""


class ParagraphsTypedDict(TypedDict):
    r"""Document paragraph."""

    id: NotRequired[str]
    r"""Paragraph ID of the document."""
    role: NotRequired[Role]
    r"""Paragraph role in the document."""
    formatting: NotRequired[FormattingTypedDict]
    r"""Paragraph formatting."""
    layout_references: NotRequired[List[LayoutReferencesTypedDict]]
    r"""An array of document paragraph parts sources in the page block structure, in the order of the text in the paragraph."""
    text: NotRequired[str]
    r"""Paragraph text."""
    list_reference: NotRequired[ListReferenceTypedDict]
    r"""Reference to list containing the paragraph."""


class Paragraphs(BaseModel):
    r"""Document paragraph."""

    id: Optional[str] = None
    r"""Paragraph ID of the document."""

    role: Optional[Role] = None
    r"""Paragraph role in the document."""

    formatting: Optional[Formatting] = None
    r"""Paragraph formatting."""

    layout_references: Annotated[
        Optional[List[LayoutReferences]], pydantic.Field(alias="layoutReferences")
    ] = None
    r"""An array of document paragraph parts sources in the page block structure, in the order of the text in the paragraph."""

    text: Optional[str] = None
    r"""Paragraph text."""

    list_reference: Annotated[
        Optional[ListReference], pydantic.Field(alias="listReference")
    ] = None
    r"""Reference to list containing the paragraph."""


class NumberingStyle(str, Enum):
    r"""List numbering style."""

    NONE = "None"
    DECIMAL = "Decimal"
    UPPER_ROMAN = "UpperRoman"
    LOWER_ROMAN = "LowerRoman"
    UPPER_LETTER = "UpperLetter"
    LOWER_LETTER = "LowerLetter"
    ORDINAL = "Ordinal"
    CARDINAL_TEXT = "CardinalText"
    ORDINAL_TEXT = "OrdinalText"
    HEX = "Hex"
    CHICAGO = "Chicago"
    IDEOGRAPH_DIGITAL = "IdeographDigital"
    JAPANESE_COUNTING = "JapaneseCounting"
    AIUEO = "Aiueo"
    IROHA = "Iroha"
    DECIMAL_FULL_WIDTH = "DecimalFullWidth"
    DECIMAL_HALF_WIDTH = "DecimalHalfWidth"
    JAPANESE_LEGAL = "JapaneseLegal"
    JAPANESE_DIGITAL_TEN_THOUSAND = "JapaneseDigitalTenThousand"
    DECIMAL_ENCLOSED_CIRCLE = "DecimalEnclosedCircle"
    DECIMAL_FULL_WIDTH2 = "DecimalFullWidth2"
    AIUEO_FULL_WIDTH = "AiueoFullWidth"
    IROHA_FULL_WIDTH = "IrohaFullWidth"
    DECIMAL_ZERO = "DecimalZero"
    BULLET = "Bullet"
    GANADA = "Ganada"
    CHOSUNG = "Chosung"
    DECIMAL_ENCLOSED_FULLSTOP = "DecimalEnclosedFullstop"
    DECIMAL_ENCLOSED_PAREN = "DecimalEnclosedParen"
    DECIMAL_ENCLOSED_CIRCLE_CHINESE = "DecimalEnclosedCircleChinese"
    IDEOGRAPH_ENCLOSED_CIRCLE = "IdeographEnclosedCircle"
    IDEOGRAPH_TRADITIONAL = "IdeographTraditional"
    IDEOGRAPH_ZODIAC = "IdeographZodiac"
    IDEOGRAPH_ZODIAC_TRADITIONAL = "IdeographZodiacTraditional"
    TAIWANESE_COUNTING = "TaiwaneseCounting"
    IDEOGRAPH_LEGAL_TRADITIONAL = "IdeographLegalTraditional"
    TAIWANESE_COUNTING_THOUSAND = "TaiwaneseCountingThousand"
    TAIWANESE_DIGITAL = "TaiwaneseDigital"
    CHINESE_COUNTING = "ChineseCounting"
    CHINESE_LEGAL_SIMPLIFIED = "ChineseLegalSimplified"
    CHINESE_COUNTING_THOUSAND = "ChineseCountingThousand"
    APPLICATION_DEFINED = "ApplicationDefined"
    KOREAN_DIGITAL = "KoreanDigital"
    KOREAN_COUNTING = "KoreanCounting"
    KOREAN_LEGAL = "KoreanLegal"
    KOREAN_DIGITAL2 = "KoreanDigital2"
    HEBREW1 = "Hebrew1"
    ARABIC_ALPHA = "ArabicAlpha"
    HEBREW2 = "Hebrew2"
    ARABIC_ABJAD = "ArabicAbjad"
    HINDI_VOWELS = "HindiVowels"
    HINDI_CONSONANTS = "HindiConsonants"
    HINDI_NUMBERS = "HindiNumbers"
    HINDI_COUNTING = "HindiCounting"
    THAI_LETTERS = "ThaiLetters"
    THAI_NUMBERS = "ThaiNumbers"
    THAI_COUNTING = "ThaiCounting"
    VIETNAMESE_COUNTING = "VietnameseCounting"
    NUMBER_IN_DASH = "NumberInDash"
    RUSSIAN_LOWER = "RussianLower"
    RUSSIAN_UPPER = "RussianUpper"
    BURMESE = "Burmese"
    UNNUMBERED = "Unnumbered"


class ListLevelsTypedDict(TypedDict):
    r"""List level desctiption"""

    level_index: int
    r"""Level index."""
    numbering_style: NumberingStyle
    r"""List numbering style."""
    start_number: int
    r"""Start number."""


class ListLevels(BaseModel):
    r"""List level desctiption"""

    level_index: Annotated[int, pydantic.Field(alias="levelIndex")]
    r"""Level index."""

    numbering_style: Annotated[NumberingStyle, pydantic.Field(alias="numberingStyle")]
    r"""List numbering style."""

    start_number: Annotated[int, pydantic.Field(alias="startNumber")]
    r"""Start number."""


class ListsTypedDict(TypedDict):
    r"""List description."""

    id: NotRequired[str]
    r"""List ID in the document."""
    list_levels: NotRequired[List[ListLevelsTypedDict]]
    r"""An array of list level descriptions."""


class Lists(BaseModel):
    r"""List description."""

    id: Optional[str] = None
    r"""List ID in the document."""

    list_levels: Annotated[
        Optional[List[ListLevels]], pydantic.Field(alias="listLevels")
    ] = None
    r"""An array of list level descriptions."""


class ContentTypedDict(TypedDict):
    r"""Document content (logical structure)."""

    paragraphs: NotRequired[List[ParagraphsTypedDict]]
    r"""An array of document paragraphs ordered by reading direction."""
    lists: NotRequired[List[ListsTypedDict]]
    r"""An array of lists found in the document."""


class Content(BaseModel):
    r"""Document content (logical structure)."""

    paragraphs: Optional[List[Paragraphs]] = None
    r"""An array of document paragraphs ordered by reading direction."""

    lists: Optional[List[Lists]] = None
    r"""An array of lists found in the document."""


class TextTypedDict(TypedDict):
    languages: List[str]
    r"""List of document languages."""
    layout: NotRequired[LayoutTypedDict]
    r"""Layout (physical structure)."""
    content: NotRequired[ContentTypedDict]
    r"""Document content (logical structure)."""


class Text(BaseModel):
    languages: List[str]
    r"""List of document languages."""

    layout: Optional[Layout] = None
    r"""Layout (physical structure)."""

    content: Optional[Content] = None
    r"""Document content (logical structure)."""


class ExtractedTextTypedDict(TypedDict):
    r"""The response from the `getExtractedText` endpoint."""

    meta: DocumentTypedDict
    r"""Represents a document that has been submitted for processing."""
    text: NotRequired[TextTypedDict]


class ExtractedText(BaseModel):
    r"""The response from the `getExtractedText` endpoint."""

    meta: Document
    r"""Represents a document that has been submitted for processing."""

    text: Optional[Text] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from abbyy_document_ai.types import BaseModel
from datetime import date
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class SeaWaybillFieldsShipperTypedDict(TypedDict):
    account_number: NotRequired[str]
    r"""The shipper's account number"""
    name: NotRequired[str]
    r"""The shipper's name"""
    address: NotRequired[str]
    r"""The shipper's address"""
    country: NotRequired[str]
    r"""The shipper's country"""
    state_or_province: NotRequired[str]
    r"""The shipper's state or province"""
    city: NotRequired[str]
    r"""The shipper's city"""
    postal_code: NotRequired[str]
    r"""The shipper's postal code"""
    street: NotRequired[str]
    r"""The shipper's street address"""
    tax_id: NotRequired[str]
    r"""The shipper's international tax identification number"""


class SeaWaybillFieldsShipper(BaseModel):
    account_number: Annotated[Optional[str], pydantic.Field(alias="accountNumber")] = (
        None
    )
    r"""The shipper's account number"""

    name: Optional[str] = None
    r"""The shipper's name"""

    address: Optional[str] = None
    r"""The shipper's address"""

    country: Optional[str] = None
    r"""The shipper's country"""

    state_or_province: Annotated[
        Optional[str], pydantic.Field(alias="stateOrProvince")
    ] = None
    r"""The shipper's state or province"""

    city: Optional[str] = None
    r"""The shipper's city"""

    postal_code: Annotated[Optional[str], pydantic.Field(alias="postalCode")] = None
    r"""The shipper's postal code"""

    street: Optional[str] = None
    r"""The shipper's street address"""

    tax_id: Annotated[Optional[str], pydantic.Field(alias="taxId")] = None
    r"""The shipper's international tax identification number"""


class SeaWaybillFieldsConsigneeTypedDict(TypedDict):
    account_number: NotRequired[str]
    r"""The consignee's account number"""
    name: NotRequired[str]
    r"""The consignee's name"""
    address: NotRequired[str]
    r"""The consignee's address"""
    country: NotRequired[str]
    r"""The consignee's country"""
    state_or_province: NotRequired[str]
    r"""The consignee's state or province"""
    city: NotRequired[str]
    r"""The consignee's city"""
    postal_code: NotRequired[str]
    r"""The consignee's postal code"""
    street: NotRequired[str]
    r"""The consignee's street address"""
    tax_id: NotRequired[str]
    r"""The consignee's international tax identification number"""


class SeaWaybillFieldsConsignee(BaseModel):
    account_number: Annotated[Optional[str], pydantic.Field(alias="accountNumber")] = (
        None
    )
    r"""The consignee's account number"""

    name: Optional[str] = None
    r"""The consignee's name"""

    address: Optional[str] = None
    r"""The consignee's address"""

    country: Optional[str] = None
    r"""The consignee's country"""

    state_or_province: Annotated[
        Optional[str], pydantic.Field(alias="stateOrProvince")
    ] = None
    r"""The consignee's state or province"""

    city: Optional[str] = None
    r"""The consignee's city"""

    postal_code: Annotated[Optional[str], pydantic.Field(alias="postalCode")] = None
    r"""The consignee's postal code"""

    street: Optional[str] = None
    r"""The consignee's street address"""

    tax_id: Annotated[Optional[str], pydantic.Field(alias="taxId")] = None
    r"""The consignee's international tax identification number"""


class SeaWaybillFieldsNotifyPartyTypedDict(TypedDict):
    name: NotRequired[str]
    r"""The name of the company to be notified of the arrival of the cargo"""
    address: NotRequired[str]
    r"""The address of the company to be notified of the arrival of the cargo"""
    country: NotRequired[str]
    r"""The country of the company to be notified"""
    state_or_province: NotRequired[str]
    r"""The state or province of the company to be notified"""
    city: NotRequired[str]
    r"""The city of the company to be notified"""
    postal_code: NotRequired[str]
    r"""The postal code of the company to be notified"""
    street: NotRequired[str]
    r"""The street address of the company to be notified"""
    tax_id: NotRequired[str]
    r"""The international tax identification number of the company to be notified"""


class SeaWaybillFieldsNotifyParty(BaseModel):
    name: Optional[str] = None
    r"""The name of the company to be notified of the arrival of the cargo"""

    address: Optional[str] = None
    r"""The address of the company to be notified of the arrival of the cargo"""

    country: Optional[str] = None
    r"""The country of the company to be notified"""

    state_or_province: Annotated[
        Optional[str], pydantic.Field(alias="stateOrProvince")
    ] = None
    r"""The state or province of the company to be notified"""

    city: Optional[str] = None
    r"""The city of the company to be notified"""

    postal_code: Annotated[Optional[str], pydantic.Field(alias="postalCode")] = None
    r"""The postal code of the company to be notified"""

    street: Optional[str] = None
    r"""The street address of the company to be notified"""

    tax_id: Annotated[Optional[str], pydantic.Field(alias="taxId")] = None
    r"""The international tax identification number of the company to be notified"""


class SeaWaybillFieldsContainersTypedDict(TypedDict):
    container_number: NotRequired[str]
    r"""The unique number of the container"""
    seal_number: NotRequired[str]
    r"""The unique number of the container seal"""
    container_type: NotRequired[str]
    r"""The type or size of the container (for example, 20-foot, 40-foot, or 20GP)"""
    quantity: NotRequired[float]
    r"""The number of packages loaded into the container"""
    quantity_uom: NotRequired[str]
    r"""The type of packages loaded into the container"""
    net_weight_kg: NotRequired[float]
    r"""The net weight of the container in kilograms"""
    gross_weight_kg: NotRequired[float]
    r"""The gross weight of the container in kilograms"""
    gross_weight_lb: NotRequired[float]
    r"""The gross weight of the container in pounds"""
    volume_m3: NotRequired[float]
    r"""The volume of the container in cubic meters"""
    volume_ft3: NotRequired[float]
    r"""The volume of the container in cubic feet"""


class SeaWaybillFieldsContainers(BaseModel):
    container_number: Annotated[
        Optional[str], pydantic.Field(alias="containerNumber")
    ] = None
    r"""The unique number of the container"""

    seal_number: Annotated[Optional[str], pydantic.Field(alias="sealNumber")] = None
    r"""The unique number of the container seal"""

    container_type: Annotated[Optional[str], pydantic.Field(alias="containerType")] = (
        None
    )
    r"""The type or size of the container (for example, 20-foot, 40-foot, or 20GP)"""

    quantity: Optional[float] = None
    r"""The number of packages loaded into the container"""

    quantity_uom: Annotated[Optional[str], pydantic.Field(alias="quantityUom")] = None
    r"""The type of packages loaded into the container"""

    net_weight_kg: Annotated[Optional[float], pydantic.Field(alias="netWeightKg")] = (
        None
    )
    r"""The net weight of the container in kilograms"""

    gross_weight_kg: Annotated[
        Optional[float], pydantic.Field(alias="grossWeightKg")
    ] = None
    r"""The gross weight of the container in kilograms"""

    gross_weight_lb: Annotated[
        Optional[float], pydantic.Field(alias="grossWeightLb")
    ] = None
    r"""The gross weight of the container in pounds"""

    volume_m3: Annotated[Optional[float], pydantic.Field(alias="volumeM3")] = None
    r"""The volume of the container in cubic meters"""

    volume_ft3: Annotated[Optional[float], pydantic.Field(alias="volumeFt3")] = None
    r"""The volume of the container in cubic feet"""


class SeaWaybillFieldsLineItemsTypedDict(TypedDict):
    marks_and_numbers: NotRequired[str]
    r"""The numbers of the shipping containers used for the goods"""
    quantity: NotRequired[float]
    r"""The number of pieces"""
    quantity_uom: NotRequired[str]
    r"""The unit of measurement used for the goods (bulk, bags, palettes, etc.)"""
    description: NotRequired[str]
    r"""A description of the goods"""
    gross_weight_kg: NotRequired[float]
    r"""The gross weight of the goods in kilograms"""
    gross_weight_lb: NotRequired[float]
    r"""The gross weight of the goods in pounds"""
    volume_m3: NotRequired[float]
    r"""The volume of the goods in cubic meters"""
    volume_ft3: NotRequired[float]
    r"""The volume of the goods in cubic feet"""


class SeaWaybillFieldsLineItems(BaseModel):
    marks_and_numbers: Annotated[
        Optional[str], pydantic.Field(alias="marksAndNumbers")
    ] = None
    r"""The numbers of the shipping containers used for the goods"""

    quantity: Optional[float] = None
    r"""The number of pieces"""

    quantity_uom: Annotated[Optional[str], pydantic.Field(alias="quantityUom")] = None
    r"""The unit of measurement used for the goods (bulk, bags, palettes, etc.)"""

    description: Optional[str] = None
    r"""A description of the goods"""

    gross_weight_kg: Annotated[
        Optional[float], pydantic.Field(alias="grossWeightKg")
    ] = None
    r"""The gross weight of the goods in kilograms"""

    gross_weight_lb: Annotated[
        Optional[float], pydantic.Field(alias="grossWeightLb")
    ] = None
    r"""The gross weight of the goods in pounds"""

    volume_m3: Annotated[Optional[float], pydantic.Field(alias="volumeM3")] = None
    r"""The volume of the goods in cubic meters"""

    volume_ft3: Annotated[Optional[float], pydantic.Field(alias="volumeFt3")] = None
    r"""The volume of the goods in cubic feet"""


class SeaWaybillFieldsTotalsTypedDict(TypedDict):
    total_quantity: NotRequired[float]
    r"""The total number of pieces"""
    total_quantity_uom: NotRequired[str]
    r"""The unit of measurement used for the goods (bulk, bags, palettes, etc.)"""
    total_weight_kg: NotRequired[float]
    r"""The total weight of the goods in kilograms"""
    total_volume_m3: NotRequired[float]
    r"""The total volume of the goods in cubic meters"""
    total_number_of_containers: NotRequired[float]
    r"""The total number of containers"""
    total_net_weight_kg: NotRequired[float]
    r"""The total net weight of the goods in kilograms"""


class SeaWaybillFieldsTotals(BaseModel):
    total_quantity: Annotated[
        Optional[float], pydantic.Field(alias="totalQuantity")
    ] = None
    r"""The total number of pieces"""

    total_quantity_uom: Annotated[
        Optional[str], pydantic.Field(alias="totalQuantityUom")
    ] = None
    r"""The unit of measurement used for the goods (bulk, bags, palettes, etc.)"""

    total_weight_kg: Annotated[
        Optional[float], pydantic.Field(alias="totalWeightKg")
    ] = None
    r"""The total weight of the goods in kilograms"""

    total_volume_m3: Annotated[
        Optional[float], pydantic.Field(alias="totalVolumeM3")
    ] = None
    r"""The total volume of the goods in cubic meters"""

    total_number_of_containers: Annotated[
        Optional[float], pydantic.Field(alias="totalNumberOfContainers")
    ] = None
    r"""The total number of containers"""

    total_net_weight_kg: Annotated[
        Optional[float], pydantic.Field(alias="totalNetWeightKg")
    ] = None
    r"""The total net weight of the goods in kilograms"""


class FreightAndChargesTypedDict(TypedDict):
    total_prepaid: NotRequired[float]
    r"""The total amount of prepaid charges"""
    currency_prepaid: NotRequired[str]
    r"""The currency of the prepaid charges"""
    total_collect: NotRequired[float]
    r"""The amount payable upon receipt"""
    currency_collect: NotRequired[str]
    r"""The currency of the amount payable upon receipt"""


class FreightAndCharges(BaseModel):
    total_prepaid: Annotated[Optional[float], pydantic.Field(alias="totalPrepaid")] = (
        None
    )
    r"""The total amount of prepaid charges"""

    currency_prepaid: Annotated[
        Optional[str], pydantic.Field(alias="currencyPrepaid")
    ] = None
    r"""The currency of the prepaid charges"""

    total_collect: Annotated[Optional[float], pydantic.Field(alias="totalCollect")] = (
        None
    )
    r"""The amount payable upon receipt"""

    currency_collect: Annotated[
        Optional[str], pydantic.Field(alias="currencyCollect")
    ] = None
    r"""The currency of the amount payable upon receipt"""


class SeaWaybillFieldsTypedDict(TypedDict):
    r"""Fields extracted from a sea waybill"""

    shipper: NotRequired[SeaWaybillFieldsShipperTypedDict]
    consignee: NotRequired[SeaWaybillFieldsConsigneeTypedDict]
    notify_party: NotRequired[SeaWaybillFieldsNotifyPartyTypedDict]
    carrier: NotRequired[str]
    r"""The carrier's name"""
    also_notify_party: NotRequired[str]
    r"""A second party that is notified of goods coming forward for delivery"""
    waybill_number: NotRequired[str]
    r"""The sea waybill number"""
    scac: NotRequired[str]
    r"""The Standard Carrier Alpha Code"""
    ico_number: NotRequired[List[str]]
    vessel_name: NotRequired[str]
    r"""The name of the vessel engaged to carry the goods"""
    voyage_number: NotRequired[str]
    r"""The number of the voyage carrying the goods"""
    port_of_loading: NotRequired[str]
    r"""The port where the goods are to be loaded"""
    port_of_discharge: NotRequired[str]
    r"""The port where the goods are to be discharged"""
    pre_carried_by: NotRequired[str]
    r"""Information about the previous carrier (if any)"""
    place_of_receipt: NotRequired[str]
    r"""The place of receipt of the goods"""
    place_of_delivery: NotRequired[str]
    r"""The place of delivery of the goods"""
    bill_of_lading_number: NotRequired[str]
    r"""The number of the bill of lading accompanying the goods"""
    booking_no: NotRequired[str]
    r"""The booking number"""
    reference: NotRequired[str]
    r"""A reference to the accompanying documents in the customers' databases"""
    origin_of_goods: NotRequired[str]
    r"""The country where the goods were manufactured"""
    containers: NotRequired[List[SeaWaybillFieldsContainersTypedDict]]
    line_items: NotRequired[List[SeaWaybillFieldsLineItemsTypedDict]]
    totals: NotRequired[SeaWaybillFieldsTotalsTypedDict]
    freight_and_charges: NotRequired[FreightAndChargesTypedDict]
    place_of_issue_of_waybill: NotRequired[str]
    r"""The place of issue of the sea waybill"""
    shipped_on_board_date: NotRequired[date]
    r"""The date of loading the cargo"""
    date_of_waybill: NotRequired[date]
    r"""The date of the sea waybill"""


class SeaWaybillFields(BaseModel):
    r"""Fields extracted from a sea waybill"""

    shipper: Optional[SeaWaybillFieldsShipper] = None

    consignee: Optional[SeaWaybillFieldsConsignee] = None

    notify_party: Annotated[
        Optional[SeaWaybillFieldsNotifyParty], pydantic.Field(alias="notifyParty")
    ] = None

    carrier: Optional[str] = None
    r"""The carrier's name"""

    also_notify_party: Annotated[
        Optional[str], pydantic.Field(alias="alsoNotifyParty")
    ] = None
    r"""A second party that is notified of goods coming forward for delivery"""

    waybill_number: Annotated[Optional[str], pydantic.Field(alias="waybillNumber")] = (
        None
    )
    r"""The sea waybill number"""

    scac: Optional[str] = None
    r"""The Standard Carrier Alpha Code"""

    ico_number: Annotated[Optional[List[str]], pydantic.Field(alias="icoNumber")] = None

    vessel_name: Annotated[Optional[str], pydantic.Field(alias="vesselName")] = None
    r"""The name of the vessel engaged to carry the goods"""

    voyage_number: Annotated[Optional[str], pydantic.Field(alias="voyageNumber")] = None
    r"""The number of the voyage carrying the goods"""

    port_of_loading: Annotated[Optional[str], pydantic.Field(alias="portOfLoading")] = (
        None
    )
    r"""The port where the goods are to be loaded"""

    port_of_discharge: Annotated[
        Optional[str], pydantic.Field(alias="portOfDischarge")
    ] = None
    r"""The port where the goods are to be discharged"""

    pre_carried_by: Annotated[Optional[str], pydantic.Field(alias="preCarriedBy")] = (
        None
    )
    r"""Information about the previous carrier (if any)"""

    place_of_receipt: Annotated[
        Optional[str], pydantic.Field(alias="placeOfReceipt")
    ] = None
    r"""The place of receipt of the goods"""

    place_of_delivery: Annotated[
        Optional[str], pydantic.Field(alias="placeOfDelivery")
    ] = None
    r"""The place of delivery of the goods"""

    bill_of_lading_number: Annotated[
        Optional[str], pydantic.Field(alias="billOfLadingNumber")
    ] = None
    r"""The number of the bill of lading accompanying the goods"""

    booking_no: Annotated[Optional[str], pydantic.Field(alias="bookingNo")] = None
    r"""The booking number"""

    reference: Optional[str] = None
    r"""A reference to the accompanying documents in the customers' databases"""

    origin_of_goods: Annotated[Optional[str], pydantic.Field(alias="originOfGoods")] = (
        None
    )
    r"""The country where the goods were manufactured"""

    containers: Optional[List[SeaWaybillFieldsContainers]] = None

    line_items: Annotated[
        Optional[List[SeaWaybillFieldsLineItems]], pydantic.Field(alias="lineItems")
    ] = None

    totals: Optional[SeaWaybillFieldsTotals] = None

    freight_and_charges: Annotated[
        Optional[FreightAndCharges], pydantic.Field(alias="freightAndCharges")
    ] = None

    place_of_issue_of_waybill: Annotated[
        Optional[str], pydantic.Field(alias="placeOfIssueOfWaybill")
    ] = None
    r"""The place of issue of the sea waybill"""

    shipped_on_board_date: Annotated[
        Optional[date], pydantic.Field(alias="shippedOnBoardDate")
    ] = None
    r"""The date of loading the cargo"""

    date_of_waybill: Annotated[
        Optional[date], pydantic.Field(alias="dateOfWaybill")
    ] = None
    r"""The date of the sea waybill"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from abbyy_document_ai.types import BaseModel
from datetime import date
from enum import Enum
import pydantic
from typing import List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


class TaxiReceiptFieldsTaxesTypedDict(TypedDict):
    tax_rate: NotRequired[float]
    r"""The tax rate applied to the purchase."""
    tax_amount: NotRequired[float]
    r"""The amount of tax for this rate."""


class TaxiReceiptFieldsTaxes(BaseModel):
    tax_rate: Annotated[Optional[float], pydantic.Field(alias="taxRate")] = None
    r"""The tax rate applied to the purchase."""

    tax_amount: Annotated[Optional[float], pydantic.Field(alias="taxAmount")] = None
    r"""The amount of tax for this rate."""


class PaymentMethod5(str, Enum):
    UNDEFINED = "Undefined"


class PaymentMethod4(str, Enum):
    MIXED = "Mixed"


class PaymentMethod3(str, Enum):
    TRANSFER = "Transfer"


class PaymentMethod2(str, Enum):
    CASH = "Cash"


class PaymentMethod1(str, Enum):
    CARD = "Card"


TaxiReceiptFieldsPaymentMethodTypedDict = TypeAliasType(
    "TaxiReceiptFieldsPaymentMethodTypedDict",
    Union[
        PaymentMethod1, PaymentMethod2, PaymentMethod3, PaymentMethod4, PaymentMethod5
    ],
)
r"""Payment method used for the transaction."""


TaxiReceiptFieldsPaymentMethod = TypeAliasType(
    "TaxiReceiptFieldsPaymentMethod",
    Union[
        PaymentMethod1, PaymentMethod2, PaymentMethod3, PaymentMethod4, PaymentMethod5
    ],
)
r"""Payment method used for the transaction."""


class TaxiReceiptFieldsCardType5(str, Enum):
    UNDEFINED = "Undefined"


class TaxiReceiptFieldsCardType4(str, Enum):
    NATIONAL_CARD = "NationalCard"


class TaxiReceiptFieldsCardType3(str, Enum):
    AMERICAN_EXPRESS = "AmericanExpress"


class TaxiReceiptFieldsCardType2(str, Enum):
    VISA = "VISA"


class TaxiReceiptFieldsCardType1(str, Enum):
    MASTERCARD = "Mastercard"


TaxiReceiptFieldsCardTypeTypedDict = TypeAliasType(
    "TaxiReceiptFieldsCardTypeTypedDict",
    Union[
        TaxiReceiptFieldsCardType1,
        TaxiReceiptFieldsCardType2,
        TaxiReceiptFieldsCardType3,
        TaxiReceiptFieldsCardType4,
        TaxiReceiptFieldsCardType5,
    ],
)
r"""The payment processing network of the card used for payment."""


TaxiReceiptFieldsCardType = TypeAliasType(
    "TaxiReceiptFieldsCardType",
    Union[
        TaxiReceiptFieldsCardType1,
        TaxiReceiptFieldsCardType2,
        TaxiReceiptFieldsCardType3,
        TaxiReceiptFieldsCardType4,
        TaxiReceiptFieldsCardType5,
    ],
)
r"""The payment processing network of the card used for payment."""


class TaxiReceiptFieldsCreditOrDebit2(str, Enum):
    DEBIT = "Debit"


class TaxiReceiptFieldsCreditOrDebit1(str, Enum):
    CREDIT = "Credit"


TaxiReceiptFieldsCreditOrDebitTypedDict = TypeAliasType(
    "TaxiReceiptFieldsCreditOrDebitTypedDict",
    Union[TaxiReceiptFieldsCreditOrDebit1, TaxiReceiptFieldsCreditOrDebit2],
)
r"""Whether a credit or debit card was used."""


TaxiReceiptFieldsCreditOrDebit = TypeAliasType(
    "TaxiReceiptFieldsCreditOrDebit",
    Union[TaxiReceiptFieldsCreditOrDebit1, TaxiReceiptFieldsCreditOrDebit2],
)
r"""Whether a credit or debit card was used."""


class TaxiReceiptFieldsTypedDict(TypedDict):
    r"""Fields extracted from a taxi receipt"""

    date_: NotRequired[date]
    r"""The date when the receipt was issued by the service provider."""
    start_date: NotRequired[date]
    r"""The date when the taxi ride started."""
    end_date: NotRequired[date]
    r"""The date when the taxi ride ended."""
    days: NotRequired[float]
    r"""The number of days for which the taxi services were provided."""
    from_location: NotRequired[str]
    r"""The starting location of the taxi ride."""
    to_location: NotRequired[str]
    r"""The destination of the taxi ride."""
    vendor: NotRequired[str]
    r"""The service provider responsible for the transportation service."""
    address: NotRequired[str]
    r"""The address of the service provider."""
    city: NotRequired[str]
    r"""The city where the service provider is located."""
    state: NotRequired[str]
    r"""The state/province where the service provider is located."""
    postal_code: NotRequired[str]
    r"""The postal code of the service provider's address."""
    country: NotRequired[str]
    r"""The country where the service provider is located."""
    phone: NotRequired[str]
    r"""The phone number of the service provider."""
    currency: NotRequired[str]
    r"""The currency of the payment."""
    total: NotRequired[float]
    r"""The total amount of the receipt."""
    tax_total: NotRequired[float]
    r"""The total amount of tax levied on the purchase."""
    taxes: NotRequired[List[TaxiReceiptFieldsTaxesTypedDict]]
    payment_method: NotRequired[TaxiReceiptFieldsPaymentMethodTypedDict]
    r"""Payment method used for the transaction."""
    card_type: NotRequired[TaxiReceiptFieldsCardTypeTypedDict]
    r"""The payment processing network of the card used for payment."""
    credit_or_debit: NotRequired[TaxiReceiptFieldsCreditOrDebitTypedDict]
    r"""Whether a credit or debit card was used."""
    card_number: NotRequired[str]
    r"""The number of the card used for payment. May be partially masked."""


class TaxiReceiptFields(BaseModel):
    r"""Fields extracted from a taxi receipt"""

    date_: Annotated[Optional[date], pydantic.Field(alias="date")] = None
    r"""The date when the receipt was issued by the service provider."""

    start_date: Annotated[Optional[date], pydantic.Field(alias="startDate")] = None
    r"""The date when the taxi ride started."""

    end_date: Annotated[Optional[date], pydantic.Field(alias="endDate")] = None
    r"""The date when the taxi ride ended."""

    days: Optional[float] = None
    r"""The number of days for which the taxi services were provided."""

    from_location: Annotated[Optional[str], pydantic.Field(alias="fromLocation")] = None
    r"""The starting location of the taxi ride."""

    to_location: Annotated[Optional[str], pydantic.Field(alias="toLocation")] = None
    r"""The destination of the taxi ride."""

    vendor: Optional[str] = None
    r"""The service provider responsible for the transportation service."""

    address: Optional[str] = None
    r"""The address of the service provider."""

    city: Optional[str] = None
    r"""The city where the service provider is located."""

    state: Optional[str] = None
    r"""The state/province where the service provider is located."""

    postal_code: Annotated[Optional[str], pydantic.Field(alias="postalCode")] = None
    r"""The postal code of the service provider's address."""

    country: Optional[str] = None
    r"""The country where the service provider is located."""

    phone: Optional[str] = None
    r"""The phone number of the service provider."""

    currency: Optional[str] = None
    r"""The currency of the payment."""

    total: Optional[float] = None
    r"""The total amount of the receipt."""

    tax_total: Annotated[Optional[float], pydantic.Field(alias="taxTotal")] = None
    r"""The total amount of tax levied on the purchase."""

    taxes: Optional[List[TaxiReceiptFieldsTaxes]] = None

    payment_method: Annotated[
        Optional[TaxiReceiptFieldsPaymentMethod], pydantic.Field(alias="paymentMethod")
    ] = None
    r"""Payment method used for the transaction."""

    card_type: Annotated[
        Optional[TaxiReceiptFieldsCardType], pydantic.Field(alias="cardType")
    ] = None
    r"""The payment processing network of the card used for payment."""

    credit_or_debit: Annotated[
        Optional[TaxiReceiptFieldsCreditOrDebit], pydantic.Field(alias="creditOrDebit")
    ] = None
    r"""Whether a credit or debit card was used."""

    card_number: Annotated[Optional[str], pydantic.Field(alias="cardNumber")] = None
    r"""The number of the card used for payment. May be partially masked."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from abbyy_document_ai import utils
from abbyy_document_ai.types import BaseModel
from enum import Enum
import pydantic
from typing import Any, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


class ElevenTypedDict(TypedDict):
    allowed_values: List[str]


class Eleven(BaseModel):
    allowed_values: Annotated[List[str], pydantic.Field(alias="allowedValues")]


class TenTypedDict(TypedDict):
    error: str
    tag: str
    tag_value: str


class Ten(BaseModel):
    error: str

    tag: str

    tag_value: Annotated[str, pydantic.Field(alias="tagValue")]


class NineTypedDict(TypedDict):
    property_name: str


class Nine(BaseModel):
    property_name: Annotated[str, pydantic.Field(alias="propertyName")]


class EightTypedDict(TypedDict):
    missing_property: str


class Eight(BaseModel):
    missing_property: Annotated[str, pydantic.Field(alias="missingProperty")]


class SevenTypedDict(TypedDict):
    pattern: str


class Seven(BaseModel):
    pattern: str


class SixTypedDict(TypedDict):
    multiple_of: float


class Six(BaseModel):
    multiple_of: Annotated[float, pydantic.Field(alias="multipleOf")]


class Comparison4(str, Enum):
    GREATER_THAN_ = ">"


class Comparison3(str, Enum):
    LESS_THAN_ = "<"


class Comparison2(str, Enum):
    GREATER_THAN_EQUAL_ = ">="


class Comparison1(str, Enum):
    LESS_THAN_EQUAL_ = "<="


ComparisonTypedDict = TypeAliasType(
    "ComparisonTypedDict", Union[Comparison1, Comparison2, Comparison3, Comparison4]
)


Comparison = TypeAliasType(
    "Comparison", Union[Comparison1, Comparison2, Comparison3, Comparison4]
)


class Params5TypedDict(TypedDict):
    limit: float
    comparison: ComparisonTypedDict


class Params5(BaseModel):
    limit: float

    comparison: Comparison


class Params4TypedDict(TypedDict):
    format_: str


class Params4(BaseModel):
    format_: Annotated[str, pydantic.Field(alias="format")]


class Params3TypedDict(TypedDict):
    property: str
    missing_property: str
    deps: str
    deps_count: float


class Params3(BaseModel):
    property: str

    missing_property: Annotated[str, pydantic.Field(alias="missingProperty")]

    deps: str

    deps_count: Annotated[float, pydantic.Field(alias="depsCount")]


class Params2TypedDict(TypedDict):
    additional_property: str


class Params2(BaseModel):
    additional_property: Annotated[str, pydantic.Field(alias="additionalProperty")]


class Params1TypedDict(TypedDict):
    limit: float


class Params1(BaseModel):
    limit: float


ParamsTypedDict = TypeAliasType(
    "ParamsTypedDict",
    Union[
        Params1TypedDict,
        Params2TypedDict,
        Params4TypedDict,
        SixTypedDict,
        SevenTypedDict,
        EightTypedDict,
        NineTypedDict,
        ElevenTypedDict,
        Params5TypedDict,
        TenTypedDict,
        Params3TypedDict,
    ],
)


Params = TypeAliasType(
    "Params",
    Union[
        Params1,
        Params2,
        Params4,
        Six,
        Seven,
        Eight,
        Nine,
        Eleven,
        Params5,
        Ten,
        Params3,
    ],
)


class ValidationTypedDict(TypedDict):
    keyword: str
    instance_path: str
    schema_path: str
    params: ParamsTypedDict
    property_name: NotRequired[str]
    message: NotRequired[str]
    schema_: NotRequired[Any]
    parent_schema: NotRequired[Any]
    data: NotRequired[Any]


class Validation(BaseModel):
    keyword: str

    instance_path: Annotated[str, pydantic.Field(alias="instancePath")]

    schema_path: Annotated[str, pydantic.Field(alias="schemaPath")]

    params: Params

    property_name: Annotated[Optional[str], pydantic.Field(alias="propertyName")] = None

    message: Optional[str] = None

    schema_: Annotated[Optional[Any], pydantic.Field(alias="schema")] = None

    parent_schema: Annotated[Optional[Any], pydantic.Field(alias="parentSchema")] = None

    data: Optional[Any] = None


class ValidationContext(str, Enum):
    BODY = "body"
    PARAMS = "params"
    QUERY = "query"
    HEADERS = "headers"
    QUERYSTRING = "querystring"


class BadRequestErrorData(BaseModel):
    type: str
    r"""A URI reference that identifies the problem type."""

    title: str
    r"""A short, human-readable summary of the problem type."""

    status: float
    r"""The HTTP status code generated by the origin server for this occurrence of the problem."""

    detail: str
    r"""A human-readable explanation specific to this occurrence of the problem."""

    instance: Optional[str] = None
    r"""The URI that identifies the specific occurrence of the problem."""

    validation: Optional[List[Validation]] = None

    validation_context: Annotated[
        Optional[ValidationContext], pydantic.Field(alias="validationContext")
    ] = None


class BadRequestError(Exception):
    r"""The error returned when there is a request validation error."""

    data: BadRequestErrorData

    def __init__(self, data: BadRequestErrorData):
        self.data = data

    def __str__(self) -> str:
        return utils.marshal_json(self.data, BadRequestErrorData)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from abbyy_document_ai.types import BaseModel
from datetime import date
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class DeclarationTypeTypedDict(TypedDict):
    part1: NotRequired[str]
    r"""First subdivision of the declaration type code (EU, EX, CO, IM)"""
    part2: NotRequired[str]
    r"""Second subdivision of the declaration type code (A, B, C, D, E, F)"""
    part3: NotRequired[str]
    r"""Third subdivision of the declaration type code (can be blank)"""


class DeclarationType(BaseModel):
    part1: Optional[str] = None
    r"""First subdivision of the declaration type code (EU, EX, CO, IM)"""

    part2: Optional[str] = None
    r"""Second subdivision of the declaration type code (A, B, C, D, E, F)"""

    part3: Optional[str] = None
    r"""Third subdivision of the declaration type code (can be blank)"""


class FormsTypedDict(TypedDict):
    part1: NotRequired[float]
    r"""The declaration page number"""
    part2: NotRequired[float]
    r"""The total number of pages"""


class Forms(BaseModel):
    part1: Optional[float] = None
    r"""The declaration page number"""

    part2: Optional[float] = None
    r"""The total number of pages"""


class ReferenceNumberTypedDict(TypedDict):
    reference_number: str
    r"""The reference number (for tracking)"""


class ReferenceNumber(BaseModel):
    reference_number: Annotated[str, pydantic.Field(alias="referenceNumber")]
    r"""The reference number (for tracking)"""


class ConsignorTypedDict(TypedDict):
    name: NotRequired[str]
    r"""The consignor's name"""
    address: NotRequired[str]
    r"""The consignor's address"""
    eori_number: NotRequired[str]
    r"""The consignor's Economic Operators Registration and Identification (EORI) number"""


class Consignor(BaseModel):
    name: Optional[str] = None
    r"""The consignor's name"""

    address: Optional[str] = None
    r"""The consignor's address"""

    eori_number: Annotated[Optional[str], pydantic.Field(alias="eoriNumber")] = None
    r"""The consignor's Economic Operators Registration and Identification (EORI) number"""


class CustomsDeclarationFieldsConsigneeTypedDict(TypedDict):
    name: NotRequired[str]
    r"""The consignee's name"""
    address: NotRequired[str]
    r"""The consignee's address"""
    eori_number: NotRequired[str]
    r"""The consignee's Economic Operators Registration and Identification (EORI) number"""


class CustomsDeclarationFieldsConsignee(BaseModel):
    name: Optional[str] = None
    r"""The consignee's name"""

    address: Optional[str] = None
    r"""The consignee's address"""

    eori_number: Annotated[Optional[str], pydantic.Field(alias="eoriNumber")] = None
    r"""The consignee's Economic Operators Registration and Identification (EORI) number"""


class DeclarantTypedDict(TypedDict):
    name: NotRequired[str]
    r"""The declarant's name"""
    address: NotRequired[str]
    r"""The declarant's address"""
    eori_number: NotRequired[str]
    r"""The declarant's Economic Operators Registration and Identification (EORI) number"""


class Declarant(BaseModel):
    name: Optional[str] = None
    r"""The declarant's name"""

    address: Optional[str] = None
    r"""The declarant's address"""

    eori_number: Annotated[Optional[str], pydantic.Field(alias="eoriNumber")] = None
    r"""The declarant's Economic Operators Registration and Identification (EORI) number"""


class ControlByOfficeOfDispatchTypedDict(TypedDict):
    result: NotRequired[str]
    r"""Other information specified by the customs office"""
    seals_affixed: NotRequired[bool]
    r"""Whether seals were affixed"""
    seals_identity: NotRequired[str]
    r"""Identity of the seals"""
    time_limit: NotRequired[str]
    r"""Time limit for the declaration"""


class ControlByOfficeOfDispatch(BaseModel):
    result: Optional[str] = None
    r"""Other information specified by the customs office"""

    seals_affixed: Annotated[Optional[bool], pydantic.Field(alias="sealsAffixed")] = (
        None
    )
    r"""Whether seals were affixed"""

    seals_identity: Annotated[Optional[str], pydantic.Field(alias="sealsIdentity")] = (
        None
    )
    r"""Identity of the seals"""

    time_limit: Annotated[Optional[str], pydantic.Field(alias="timeLimit")] = None
    r"""Time limit for the declaration"""


class CustomsDeclarationFieldsTypedDict(TypedDict):
    r"""Fields extracted from a customs declaration"""

    movement_reference_number: NotRequired[str]
    r"""The Movement Reference Number (MRN)"""
    movement_reference_number_barcode: NotRequired[str]
    r"""The MRN barcode"""
    issuing_date: NotRequired[date]
    r"""The date on which the declaration was issued"""
    customs_office: NotRequired[str]
    r"""The customs office identifier"""
    declaration_type: NotRequired[DeclarationTypeTypedDict]
    forms: NotRequired[List[FormsTypedDict]]
    security_declaration: NotRequired[str]
    r"""Specifies whether the declaration contains additional risk analysis documents (S or empty)"""
    items: NotRequired[float]
    r"""The number of items in the shipment"""
    total_packages: NotRequired[float]
    r"""The total number of packages"""
    reference_number: NotRequired[List[ReferenceNumberTypedDict]]
    transport_charges_method_of_payment_code: NotRequired[str]
    r"""The code for the method of payment used to cover transport charges"""
    consignor: NotRequired[ConsignorTypedDict]
    consignee: NotRequired[CustomsDeclarationFieldsConsigneeTypedDict]
    declarant: NotRequired[List[DeclarantTypedDict]]
    export_country_code: NotRequired[str]
    r"""The consignor's country code"""
    destination_country_code: NotRequired[str]
    r"""The consignee's country code"""
    countries_of_routing_codes: NotRequired[str]
    r"""The codes for countries of routing for the shipment"""
    identity_of_means_of_transport_at_departure: NotRequired[str]
    r"""The identifier for the means of transport at departure"""
    nationality_of_means_of_transport_at_departure: NotRequired[str]
    r"""The country of registration of the transport at departure"""
    container: NotRequired[str]
    r"""Specifies whether the goods are being shipped in a container (0 or 1)"""
    identity_of_means_of_transport_at_border: NotRequired[str]
    r"""The identifier for the transport shipping the goods when they are presented at the border"""
    nationality_of_means_of_transport_at_border: NotRequired[str]
    r"""The country of registration of the transport shipping the goods when they are presented at the border"""
    currency: NotRequired[str]
    r"""The currency used to specify the value of the goods in the declaration"""
    total: NotRequired[float]
    r"""The total value of the shipped goods"""
    exchange_rate: NotRequired[float]
    r"""The currency exchange rate"""
    mode_of_transport_at_border: NotRequired[str]
    r"""The mode of transport used to arrive at the border (road, rail, air, sea)"""
    internal_mode_of_transport: NotRequired[str]
    r"""The internal mode of transport of the goods (road, rail, air, sea)"""
    customs_office_of_exit: NotRequired[str]
    r"""Identifier of the customs office that released the goods"""
    location_of_goods: NotRequired[str]
    r"""The location where the goods are stored before customs clearance"""
    gross_weight_in_kg: NotRequired[float]
    r"""The total gross weight of the packaged goods (in kilograms)"""
    seal_number: NotRequired[str]
    r"""The seal number"""
    control_by_office_of_dispatch: NotRequired[ControlByOfficeOfDispatchTypedDict]


class CustomsDeclarationFields(BaseModel):
    r"""Fields extracted from a customs declaration"""

    movement_reference_number: Annotated[
        Optional[str], pydantic.Field(alias="movementReferenceNumber")
    ] = None
    r"""The Movement Reference Number (MRN)"""

    movement_reference_number_barcode: Annotated[
        Optional[str], pydantic.Field(alias="movementReferenceNumberBarcode")
    ] = None
    r"""The MRN barcode"""

    issuing_date: Annotated[Optional[date], pydantic.Field(alias="issuingDate")] = None
    r"""The date on which the declaration was issued"""

    customs_office: Annotated[Optional[str], pydantic.Field(alias="customsOffice")] = (
        None
    )
    r"""The customs office identifier"""

    declaration_type: Annotated[
        Optional[DeclarationType], pydantic.Field(alias="declarationType")
    ] = None

    forms: Optional[List[Forms]] = None

    security_declaration: Annotated[
        Optional[str], pydantic.Field(alias="securityDeclaration")
    ] = None
    r"""Specifies whether the declaration contains additional risk analysis documents (S or empty)"""

    items: Optional[float] = None
    r"""The number of items in the shipment"""

    total_packages: Annotated[
        Optional[float], pydantic.Field(alias="totalPackages")
    ] = None
    r"""The total number of packages"""

    reference_number: Annotated[
        Optional[List[ReferenceNumber]], pydantic.Field(alias="referenceNumber")
    ] = None

    transport_charges_method_of_payment_code: Annotated[
        Optional[str], pydantic.Field(alias="transportChargesMethodOfPaymentCode")
    ] = None
    r"""The code for the method of payment used to cover transport charges"""

    consignor: Optional[Consignor] = None

    consignee: Optional[CustomsDeclarationFieldsConsignee] = None

    declarant: Optional[List[Declarant]] = None

    export_country_code: Annotated[
        Optional[str], pydantic.Field(alias="exportCountryCode")
    ] = None
    r"""The consignor's country code"""

    destination_country_code: Annotated[
        Optional[str], pydantic.Field(alias="destinationCountryCode")
    ] = None
    r"""The consignee's country code"""

    countries_of_routing_codes: Annotated[
        Optional[str], pydantic.Field(alias="countriesOfRoutingCodes")
    ] = None
    r"""The codes for countries of routing for the shipment"""

    identity_of_means_of_transport_at_departure: Annotated[
        Optional[str], pydantic.Field(alias="identityOfMeansOfTransportAtDeparture")
    ] = None
    r"""The identifier for the means of transport at departure"""

    nationality_of_means_of_transport_at_departure: Annotated[
        Optional[str], pydantic.Field(alias="nationalityOfMeansOfTransportAtDeparture")
    ] = None
    r"""The country of registration of the transport at departure"""

    container: Optional[str] = None
    r"""Specifies whether the goods are being shipped in a container (0 or 1)"""

    identity_of_means_of_transport_at_border: Annotated[
        Optional[str], pydantic.Field(alias="identityOfMeansOfTransportAtBorder")
    ] = None
    r"""The identifier for the transport shipping the goods when they are presented at the border"""

    nationality_of_means_of_transport_at_border: Annotated[
        Optional[str], pydantic.Field(alias="nationalityOfMeansOfTransportAtBorder")
    ] = None
    r"""The country of registration of the transport shipping the goods when they are presented at the border"""

    currency: Optional[str] = None
    r"""The currency used to specify the value of the goods in the declaration"""

    total: Optional[float] = None
    r"""The total value of the shipped goods"""

    exchange_rate: Annotated[Optional[float], pydantic.Field(alias="exchangeRate")] = (
        None
    )
    r"""The currency exchange rate"""

    mode_of_transport_at_border: Annotated[
        Optional[str], pydantic.Field(alias="modeOfTransportAtBorder")
    ] = None
    r"""The mode of transport used to arrive at the border (road, rail, air, sea)"""

    internal_mode_of_transport: Annotated[
        Optional[str], pydantic.Field(alias="internalModeOfTransport")
    ] = None
    r"""The internal mode of transport of the goods (road, rail, air, sea)"""

    customs_office_of_exit: Annotated[
        Optional[str], pydantic.Field(alias="customsOfficeOfExit")
    ] = None
    r"""Identifier of the customs office that released the goods"""

    location_of_goods: Annotated[
        Optional[str], pydantic.Field(alias="locationOfGoods")
    ] = None
    r"""The location where the goods are stored before customs clearance"""

    gross_weight_in_kg: Annotated[
        Optional[float], pydantic.Field(alias="grossWeightInKg")
    ] = None
    r"""The total gross weight of the packaged goods (in kilograms)"""

    seal_number: Annotated[Optional[str], pydantic.Field(alias="sealNumber")] = None
    r"""The seal number"""

    control_by_office_of_dispatch: Annotated[
        Optional[ControlByOfficeOfDispatch],
        pydantic.Field(alias="controlByOfficeOfDispatch"),
    ] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from abbyy_document_ai import models, utils
from abbyy_document_ai._hooks import HookContext
from abbyy_document_ai.types import BaseModel, OptionalNullable, UNSET
from abbyy_document_ai.utils import get_security_from_env
from typing import Any, List, Mapping, Optional, Union, cast


class PurchaseOrderSDK(BaseSDK):
    def begin_field_extraction(
        self,
        *,
        request: Optional[
            Union[
                models.BeginPurchaseOrderFieldExtractionRequestBody,
                models.BeginPurchaseOrderFieldExtractionRequestBodyTypedDict,
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.BeginPurchaseOrderFieldExtractionResponse:
        r"""Begin Purchase Order Field Extraction

        This endpoint processes a document using the purchase-order model. Note that the response can include an RFC 8288-style `Link` header that may contain links to results. If multiple documents are sent for processing, there will be a link with rel='results' for each document.

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(
                request, Optional[models.BeginPurchaseOrderFieldExtractionRequestBody]
            )
        request = cast(
            Optional[models.BeginPurchaseOrderFieldExtractionRequestBody], request
        )

        req = self._build_request(
            method="POST",
            path="/v1-preview/models/purchase-order",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                True,
                "json",
                Optional[models.BeginPurchaseOrderFieldExtractionRequestBody],
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="beginPurchaseOrderFieldExtraction",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "413",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return models.BeginPurchaseOrderFieldExtractionResponse(
                documents=utils.unmarshal_json(
                    http_res.text, Optional[List[models.Document]]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.BadRequestErrorData
            )
            raise models.BadRequestError(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnauthorizedErrorData
            )
            raise models.UnauthorizedError(data=response_data)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredErrorData
            )
            raise models.PaymentRequiredError(data=response_data)
        if utils.match_response(http_res, "413", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.ContentTooLargeErrorData
            )
            raise models.ContentTooLargeError(data=response_data)
        if utils.match_response(http_res, "415", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnsupportedMediaTypeErrorData
            )
            raise models.UnsupportedMediaTypeError(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.ProblemDetailData
            )
            raise models.ProblemDetail(data=response_data)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.TooManyRequestsErrorData
            )
            raise models.TooManyRequestsError(data=response_data)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.InternalServerErrorData
            )
            raise models.InternalServerError(data=response_data)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.ProblemDetailData
            )
            raise models.ProblemDetail(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def begin_field_extraction_async(
        self,
        *,
        request: Optional[
            Union[
                models.BeginPurchaseOrderFieldExtractionRequestBody,
                models.BeginPurchaseOrderFieldExtractionRequestBodyTypedDict,
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.BeginPurchaseOrderFieldExtractionResponse:
        r"""Begin Purchase Order Field Extraction

        This endpoint processes a document using the purchase-order model. Note that the response can include an RFC 8288-style `Link` header that may contain links to results. If multiple documents are sent for processing, there will be a link with rel='results' for each document.

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(
                request, Optional[models.BeginPurchaseOrderFieldExtractionRequestBody]
            )
        request = cast(
            Optional[models.BeginPurchaseOrderFieldExtractionRequestBody], request
        )

        req = self._build_request_async(
            method="POST",
            path="/v1-preview/models/purchase-order",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                True,
                "json",
                Optional[models.BeginPurchaseOrderFieldExtractionRequestBody],
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="beginPurchaseOrderFieldExtraction",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "413",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return models.BeginPurchaseOrderFieldExtractionResponse(
                documents=utils.unmarshal_json(
                    http_res.text, Optional[List[models.Document]]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.BadRequestErrorData
            )
            raise models.BadRequestError(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnauthorizedErrorData
            )
            raise models.UnauthorizedError(data=response_data)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredErrorData
            )
            raise models.PaymentRequiredError(data=response_data)
        if utils.match_response(http_res, "413", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.ContentTooLargeErrorData
            )
            raise models.ContentTooLargeError(data=response_data)
        if utils.match_response(http_res, "415", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnsupportedMediaTypeErrorData
            )
            raise models.UnsupportedMediaTypeError(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.ProblemDetailData
            )
            raise models.ProblemDetail(data=response_data)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.TooManyRequestsErrorData
            )
            raise models.TooManyRequestsError(data=response_data)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.InternalServerErrorData
            )
            raise models.InternalServerError(data=response_data)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.ProblemDetailData
            )
            raise models.ProblemDetail(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def get_extracted_fields(
        self,
        *,
        document_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.GetExtractedPurchaseOrderFieldsResponse:
        r"""Get Purchase Order Fields

        This endpoint returns an object representing the extracted purchase-order model fields from a document.

        :param document_id: The id of the document
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetExtractedPurchaseOrderFieldsRequest(
            document_id=document_id,
        )

        req = self._build_request(
            method="GET",
            path="/v1-preview/models/purchase-order/{documentId}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="getExtractedPurchaseOrderFields",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.GetExtractedPurchaseOrderFieldsResponse(
                purchase_order=utils.unmarshal_json(
                    http_res.text, Optional[models.PurchaseOrder]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnauthorizedErrorData
            )
            raise models.UnauthorizedError(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.NotFoundErrorData
            )
            raise models.NotFoundError(data=response_data)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.TooManyRequestsErrorData
            )
            raise models.TooManyRequestsError(data=response_data)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.InternalServerErrorData
            )
            raise models.InternalServerError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def get_extracted_fields_async(
        self,
        *,
        document_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.GetExtractedPurchaseOrderFieldsResponse:
        r"""Get Purchase Order Fields

        This endpoint returns an object representing the extracted purchase-order model fields from a document.

        :param document_id: The id of the document
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetExtractedPurchaseOrderFieldsRequest(
            document_id=document_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1-preview/models/purchase-order/{documentId}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="getExtractedPurchaseOrderFields",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.GetExtractedPurchaseOrderFieldsResponse(
                purchase_order=utils.unmarshal_json(
                    http_res.text, Optional[models.PurchaseOrder]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnauthorizedErrorData
            )
            raise models.UnauthorizedError(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.NotFoundErrorData
            )
            raise models.NotFoundError(data=response_data)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.TooManyRequestsErrorData
            )
            raise models.TooManyRequestsError(data=response_data)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.InternalServerErrorData
            )
            raise models.InternalServerError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )
